@@ -17,15 +17,14 @@
 
 import functools
 
-from oslo_config import cfg
-
 from keystone.assignment import controllers
 from keystone.common import json_home
 from keystone.common import router
 from keystone.common import wsgi
+import keystone.conf
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 
 build_os_inherit_relation = functools.partial(
     json_home.build_v3_extension_resource_relation,
@@ -63,7 +62,7 @@ class Routers(wsgi.RoutersBase):
         self._add_resource(
             mapper, project_controller,
             path='/users/{user_id}/projects',
-            get_action='list_user_projects',
+            get_head_action='list_user_projects',
             rel=json_home.build_v3_resource_relation('user_projects'),
             path_vars={
                 'user_id': json_home.Parameters.USER_ID,
@@ -137,7 +136,7 @@ class Routers(wsgi.RoutersBase):
         self._add_resource(
             mapper, grant_controller,
             path='/projects/{project_id}/users/{user_id}/roles',
-            get_action='list_grants',
+            get_head_action='list_grants',
             rel=json_home.build_v3_resource_relation('project_user_roles'),
             path_vars={
                 'project_id': json_home.Parameters.PROJECT_ID,
@@ -146,7 +145,7 @@ class Routers(wsgi.RoutersBase):
         self._add_resource(
             mapper, grant_controller,
             path='/projects/{project_id}/groups/{group_id}/roles',
-            get_action='list_grants',
+            get_head_action='list_grants',
             rel=json_home.build_v3_resource_relation('project_group_roles'),
             path_vars={
                 'group_id': json_home.Parameters.GROUP_ID,
@@ -179,7 +178,7 @@ class Routers(wsgi.RoutersBase):
         self._add_resource(
             mapper, grant_controller,
             path='/domains/{domain_id}/users/{user_id}/roles',
-            get_action='list_grants',
+            get_head_action='list_grants',
             rel=json_home.build_v3_resource_relation('domain_user_roles'),
             path_vars={
                 'domain_id': json_home.Parameters.DOMAIN_ID,
@@ -188,7 +187,7 @@ class Routers(wsgi.RoutersBase):
         self._add_resource(
             mapper, grant_controller,
             path='/domains/{domain_id}/groups/{group_id}/roles',
-            get_action='list_grants',
+            get_head_action='list_grants',
             rel=json_home.build_v3_resource_relation('domain_group_roles'),
             path_vars={
                 'domain_id': json_home.Parameters.DOMAIN_ID,
@@ -198,85 +197,84 @@ class Routers(wsgi.RoutersBase):
         self._add_resource(
             mapper, controllers.RoleAssignmentV3(),
             path='/role_assignments',
-            get_action='list_role_assignments_wrapper',
+            get_head_action='list_role_assignments_wrapper',
             rel=json_home.build_v3_resource_relation('role_assignments'))
 
-        if CONF.os_inherit.enabled:
-            self._add_resource(
-                mapper, grant_controller,
-                path='/OS-INHERIT/domains/{domain_id}/users/{user_id}/roles/'
-                '{role_id}/inherited_to_projects',
-                get_head_action='check_grant',
-                put_action='create_grant',
-                delete_action='revoke_grant',
-                rel=build_os_inherit_relation(
-                    resource_name='domain_user_role_inherited_to_projects'),
-                path_vars={
-                    'domain_id': json_home.Parameters.DOMAIN_ID,
-                    'role_id': json_home.Parameters.ROLE_ID,
-                    'user_id': json_home.Parameters.USER_ID,
-                })
-            self._add_resource(
-                mapper, grant_controller,
-                path='/OS-INHERIT/domains/{domain_id}/groups/{group_id}/roles/'
-                '{role_id}/inherited_to_projects',
-                get_head_action='check_grant',
-                put_action='create_grant',
-                delete_action='revoke_grant',
-                rel=build_os_inherit_relation(
-                    resource_name='domain_group_role_inherited_to_projects'),
-                path_vars={
-                    'domain_id': json_home.Parameters.DOMAIN_ID,
-                    'group_id': json_home.Parameters.GROUP_ID,
-                    'role_id': json_home.Parameters.ROLE_ID,
-                })
-            self._add_resource(
-                mapper, grant_controller,
-                path='/OS-INHERIT/domains/{domain_id}/groups/{group_id}/roles/'
-                'inherited_to_projects',
-                get_action='list_grants',
-                rel=build_os_inherit_relation(
-                    resource_name='domain_group_roles_inherited_to_projects'),
-                path_vars={
-                    'domain_id': json_home.Parameters.DOMAIN_ID,
-                    'group_id': json_home.Parameters.GROUP_ID,
-                })
-            self._add_resource(
-                mapper, grant_controller,
-                path='/OS-INHERIT/domains/{domain_id}/users/{user_id}/roles/'
-                'inherited_to_projects',
-                get_action='list_grants',
-                rel=build_os_inherit_relation(
-                    resource_name='domain_user_roles_inherited_to_projects'),
-                path_vars={
-                    'domain_id': json_home.Parameters.DOMAIN_ID,
-                    'user_id': json_home.Parameters.USER_ID,
-                })
-            self._add_resource(
-                mapper, grant_controller,
-                path='/OS-INHERIT/projects/{project_id}/users/{user_id}/roles/'
-                '{role_id}/inherited_to_projects',
-                get_head_action='check_grant',
-                put_action='create_grant',
-                delete_action='revoke_grant',
-                rel=build_os_inherit_relation(
-                    resource_name='project_user_role_inherited_to_projects'),
-                path_vars={
-                    'project_id': json_home.Parameters.PROJECT_ID,
-                    'user_id': json_home.Parameters.USER_ID,
-                    'role_id': json_home.Parameters.ROLE_ID,
-                })
-            self._add_resource(
-                mapper, grant_controller,
-                path='/OS-INHERIT/projects/{project_id}/groups/{group_id}/'
-                'roles/{role_id}/inherited_to_projects',
-                get_head_action='check_grant',
-                put_action='create_grant',
-                delete_action='revoke_grant',
-                rel=build_os_inherit_relation(
-                    resource_name='project_group_role_inherited_to_projects'),
-                path_vars={
-                    'project_id': json_home.Parameters.PROJECT_ID,
-                    'group_id': json_home.Parameters.GROUP_ID,
-                    'role_id': json_home.Parameters.ROLE_ID,
-                })
+        self._add_resource(
+            mapper, grant_controller,
+            path='/OS-INHERIT/domains/{domain_id}/users/{user_id}/roles/'
+            '{role_id}/inherited_to_projects',
+            get_head_action='check_grant',
+            put_action='create_grant',
+            delete_action='revoke_grant',
+            rel=build_os_inherit_relation(
+                resource_name='domain_user_role_inherited_to_projects'),
+            path_vars={
+                'domain_id': json_home.Parameters.DOMAIN_ID,
+                'role_id': json_home.Parameters.ROLE_ID,
+                'user_id': json_home.Parameters.USER_ID,
+            })
+        self._add_resource(
+            mapper, grant_controller,
+            path='/OS-INHERIT/domains/{domain_id}/groups/{group_id}/roles/'
+            '{role_id}/inherited_to_projects',
+            get_head_action='check_grant',
+            put_action='create_grant',
+            delete_action='revoke_grant',
+            rel=build_os_inherit_relation(
+                resource_name='domain_group_role_inherited_to_projects'),
+            path_vars={
+                'domain_id': json_home.Parameters.DOMAIN_ID,
+                'group_id': json_home.Parameters.GROUP_ID,
+                'role_id': json_home.Parameters.ROLE_ID,
+            })
+        self._add_resource(
+            mapper, grant_controller,
+            path='/OS-INHERIT/domains/{domain_id}/groups/{group_id}/roles/'
+            'inherited_to_projects',
+            get_action='list_grants',
+            rel=build_os_inherit_relation(
+                resource_name='domain_group_roles_inherited_to_projects'),
+            path_vars={
+                'domain_id': json_home.Parameters.DOMAIN_ID,
+                'group_id': json_home.Parameters.GROUP_ID,
+            })
+        self._add_resource(
+            mapper, grant_controller,
+            path='/OS-INHERIT/domains/{domain_id}/users/{user_id}/roles/'
+            'inherited_to_projects',
+            get_action='list_grants',
+            rel=build_os_inherit_relation(
+                resource_name='domain_user_roles_inherited_to_projects'),
+            path_vars={
+                'domain_id': json_home.Parameters.DOMAIN_ID,
+                'user_id': json_home.Parameters.USER_ID,
+            })
+        self._add_resource(
+            mapper, grant_controller,
+            path='/OS-INHERIT/projects/{project_id}/users/{user_id}/roles/'
+            '{role_id}/inherited_to_projects',
+            get_head_action='check_grant',
+            put_action='create_grant',
+            delete_action='revoke_grant',
+            rel=build_os_inherit_relation(
+                resource_name='project_user_role_inherited_to_projects'),
+            path_vars={
+                'project_id': json_home.Parameters.PROJECT_ID,
+                'user_id': json_home.Parameters.USER_ID,
+                'role_id': json_home.Parameters.ROLE_ID,
+            })
+        self._add_resource(
+            mapper, grant_controller,
+            path='/OS-INHERIT/projects/{project_id}/groups/{group_id}/'
+            'roles/{role_id}/inherited_to_projects',
+            get_head_action='check_grant',
+            put_action='create_grant',
+            delete_action='revoke_grant',
+            rel=build_os_inherit_relation(
+                resource_name='project_group_role_inherited_to_projects'),
+            path_vars={
+                'project_id': json_home.Parameters.PROJECT_ID,
+                'group_id': json_home.Parameters.GROUP_ID,
+                'role_id': json_home.Parameters.ROLE_ID,
+            })
