@@ -15,7 +15,6 @@
 import functools
 import uuid
 
-from oslo_config import cfg
 from oslo_log import log
 from oslo_log import versionutils
 from oslo_utils import strutils
@@ -26,13 +25,14 @@ from keystone.common import dependency
 from keystone.common import driver_hints
 from keystone.common import utils
 from keystone.common import wsgi
+import keystone.conf
 from keystone import exception
 from keystone.i18n import _, _LW
 from keystone.models import token_model
 
 
 LOG = log.getLogger(__name__)
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 
 
 def v2_deprecated(f):
@@ -74,41 +74,14 @@ def v2_auth_deprecated(f):
 def _build_policy_check_credentials(self, action, context, kwargs):
     kwargs_str = ', '.join(['%s=%s' % (k, kwargs[k]) for k in kwargs])
     kwargs_str = strutils.mask_password(kwargs_str)
+    msg = 'RBAC: Authorizing %(action)s(%(kwargs)s)'
+    LOG.debug(msg, {'action': action, 'kwargs': kwargs_str})
 
-    LOG.debug('RBAC: Authorizing %(action)s(%(kwargs)s)', {
-        'action': action,
-        'kwargs': kwargs_str})
-
-    # see if auth context has already been created. If so use it.
-    if ('environment' in context and
-            authorization.AUTH_CONTEXT_ENV in context['environment']):
-        LOG.debug('RBAC: using auth context from the request environment')
-        return context['environment'].get(authorization.AUTH_CONTEXT_ENV)
-
-    # There is no current auth context, build it from the incoming token.
-    # TODO(morganfainberg): Collapse this logic with AuthContextMiddleware
-    # in a sane manner as this just mirrors the logic in AuthContextMiddleware
-    try:
-        LOG.debug('RBAC: building auth context from the incoming auth token')
-        token_ref = token_model.KeystoneToken(
-            token_id=context['token_id'],
-            token_data=self.token_provider_api.validate_token(
-                context['token_id']))
-        # NOTE(jamielennox): whilst this maybe shouldn't be within this
-        # function it would otherwise need to reload the token_ref from
-        # backing store.
-        wsgi.validate_token_bind(context, token_ref)
-    except exception.TokenNotFound:
-        LOG.warning(_LW('RBAC: Invalid token'))
-        raise exception.Unauthorized()
-
-    auth_context = authorization.token_to_auth_context(token_ref)
-
-    return auth_context
+    return context['environment'].get(authorization.AUTH_CONTEXT_ENV, {})
 
 
 def protected(callback=None):
-    """Wraps API calls with role based access controls (RBAC).
+    """Wrap API calls with role based access controls (RBAC).
 
     This handles both the protection of the API parameters as well as any
     target entities for single-entity API calls.
@@ -122,17 +95,25 @@ def protected(callback=None):
     """
     def wrapper(f):
         @functools.wraps(f)
-        def inner(self, context, *args, **kwargs):
-            if 'is_admin' in context and context['is_admin']:
+        def inner(self, request, *args, **kwargs):
+            request.assert_authenticated()
+
+            if request.context.is_admin:
                 LOG.warning(_LW('RBAC: Bypassing authorization'))
             elif callback is not None:
                 prep_info = {'f_name': f.__name__,
                              'input_attr': kwargs}
-                callback(self, context, prep_info, *args, **kwargs)
+                callback(self,
+                         request,
+                         prep_info,
+                         *args,
+                         **kwargs)
             else:
                 action = 'identity:%s' % f.__name__
-                creds = _build_policy_check_credentials(self, action,
-                                                        context, kwargs)
+                creds = _build_policy_check_credentials(self,
+                                                        action,
+                                                        request.context_dict,
+                                                        kwargs)
 
                 policy_dict = {}
 
@@ -149,11 +130,13 @@ def protected(callback=None):
 
                 # TODO(henry-nash): Move this entire code to a member
                 # method inside v3 Auth
-                if context.get('subject_token_id') is not None:
+                if request.context_dict.get('subject_token_id') is not None:
+                    window_seconds = self._token_validation_window(request)
                     token_ref = token_model.KeystoneToken(
-                        token_id=context['subject_token_id'],
+                        token_id=request.context_dict['subject_token_id'],
                         token_data=self.token_provider_api.validate_token(
-                            context['subject_token_id']))
+                            request.context_dict['subject_token_id'],
+                            window_seconds=window_seconds))
                     policy_dict.setdefault('target', {})
                     policy_dict['target'].setdefault(self.member_name, {})
                     policy_dict['target'][self.member_name]['user_id'] = (
@@ -178,13 +161,13 @@ def protected(callback=None):
                                         action,
                                         utils.flatten_dict(policy_dict))
                 LOG.debug('RBAC: Authorization granted')
-            return f(self, context, *args, **kwargs)
+            return f(self, request, *args, **kwargs)
         return inner
     return wrapper
 
 
 def filterprotected(*filters, **callback):
-    """Wraps API list calls with role based access controls (RBAC).
+    """Wrap API list calls with role based access controls (RBAC).
 
     This handles both the protection of the API parameters as well as any
     filters supplied.
@@ -198,8 +181,10 @@ def filterprotected(*filters, **callback):
     """
     def _filterprotected(f):
         @functools.wraps(f)
-        def wrapper(self, context, **kwargs):
-            if not context['is_admin']:
+        def wrapper(self, request, **kwargs):
+            request.assert_authenticated()
+
+            if not request.context.is_admin:
                 # The target dict for the policy check will include:
                 #
                 # - Any query filter parameters
@@ -212,8 +197,8 @@ def filterprotected(*filters, **callback):
                 target = dict()
                 if filters:
                     for item in filters:
-                        if item in context['query_string']:
-                            target[item] = context['query_string'][item]
+                        if item in request.params:
+                            target[item] = request.params[item]
 
                     LOG.debug('RBAC: Adding query filter params (%s)', (
                         ', '.join(['%s=%s' % (item, target[item])
@@ -227,12 +212,15 @@ def filterprotected(*filters, **callback):
                     prep_info = {'f_name': f.__name__,
                                  'input_attr': kwargs,
                                  'filter_attr': target}
-                    callback['callback'](self, context, prep_info, **kwargs)
+                    callback['callback'](self,
+                                         request,
+                                         prep_info,
+                                         **kwargs)
                 else:
                     # No callback, so we are going to check the protection here
                     action = 'identity:%s' % f.__name__
-                    creds = _build_policy_check_credentials(self, action,
-                                                            context, kwargs)
+                    creds = _build_policy_check_credentials(
+                        self, action, request.context_dict, kwargs)
                     # Add in any formal url parameters
                     for key in kwargs:
                         target[key] = kwargs[key]
@@ -244,7 +232,7 @@ def filterprotected(*filters, **callback):
                     LOG.debug('RBAC: Authorization granted')
             else:
                 LOG.warning(_LW('RBAC: Bypassing authorization'))
-            return f(self, context, filters, **kwargs)
+            return f(self, request, filters, **kwargs)
         return wrapper
     return _filterprotected
 
@@ -252,7 +240,7 @@ def filterprotected(*filters, **callback):
 class V2Controller(wsgi.Application):
     """Base controller class for Identity API v2."""
 
-    def _normalize_domain_id(self, context, ref):
+    def _normalize_domain_id(self, request, ref):
         """Fill in domain_id since v2 calls are not domain-aware.
 
         This will overwrite any domain_id that was inadvertently
@@ -288,7 +276,7 @@ class V2Controller(wsgi.Application):
 
     @staticmethod
     def normalize_username_in_response(ref):
-        """Adds username to outgoing user refs to match the v2 spec.
+        """Add username to outgoing user refs to match the v2 spec.
 
         Internally we use `name` to represent a user's name. The v2 spec
         requires the use of `username` instead.
@@ -300,7 +288,7 @@ class V2Controller(wsgi.Application):
 
     @staticmethod
     def normalize_username_in_request(ref):
-        """Adds name in incoming user refs to match the v2 spec.
+        """Add name in incoming user refs to match the v2 spec.
 
         Internally we use `name` to represent a user's name. The v2 spec
         requires the use of `username` instead.
@@ -317,6 +305,7 @@ class V2Controller(wsgi.Application):
         * v2.0 users are not domain aware, and should have domain_id removed
         * v2.0 users expect the use of tenantId instead of default_project_id
         * v2.0 users have a username attribute
+        * v2.0 remove password_expires_at
 
         If ref is a list type, we will iterate through each element and do the
         conversion.
@@ -336,6 +325,7 @@ class V2Controller(wsgi.Application):
         def _normalize_and_filter_user_properties(ref):
             """Run through the various filter/normalization methods."""
             _format_default_project_id(ref)
+            ref.pop('password_expires_at', None)
             V2Controller.filter_domain(ref)
             V2Controller.filter_domain_id(ref)
             V2Controller.normalize_username_in_response(ref)
@@ -440,12 +430,6 @@ class V3Controller(wsgi.Application):
 
         return '%s/%s/%s' % (endpoint, 'v3', path.lstrip('/'))
 
-    def get_auth_context(self, context):
-        # TODO(dolphm): this method of accessing the auth context is terrible,
-        # but context needs to be refactored to always have reasonable values.
-        env_context = context.get('environment', {})
-        return env_context.get(authorization.AUTH_CONTEXT_ENV, {})
-
     @classmethod
     def full_url(cls, context, path=None):
         url = cls.base_url(context, path)
@@ -527,7 +511,7 @@ class V3Controller(wsgi.Application):
 
     @classmethod
     def limit(cls, refs, hints):
-        """Limits a list of entities.
+        """Limit a list of entities.
 
         The underlying driver layer may have already truncated the collection
         for us, but in case it was unable to handle truncation we check here.
@@ -560,9 +544,9 @@ class V3Controller(wsgi.Application):
 
     @classmethod
     def filter_by_attributes(cls, refs, hints):
-        """Filters a list of references by filter values."""
+        """Filter a list of references by filter values."""
         def _attr_match(ref_attr, val_attr):
-            """Matches attributes allowing for booleans as strings.
+            """Matche attributes allowing for booleans as strings.
 
             We test explicitly for a value that defines it as 'False',
             which also means that the existence of the attribute with
@@ -575,7 +559,7 @@ class V3Controller(wsgi.Application):
                 return ref_attr == val_attr
 
         def _inexact_attr_match(filter, ref):
-            """Applies an inexact filter to a result dict.
+            """Apply an inexact filter to a result dict.
 
             :param filter: the filter in question
             :param ref: the dict to check
@@ -621,25 +605,23 @@ class V3Controller(wsgi.Application):
         return refs
 
     @classmethod
-    def build_driver_hints(cls, context, supported_filters):
+    def build_driver_hints(cls, request, supported_filters):
         """Build list hints based on the context query string.
 
-        :param context: contains the query_string from which any list hints can
-                        be extracted
+        :param request: the current request
         :param supported_filters: list of filters supported, so ignore any
                                   keys in query_dict that are not in this list.
 
         """
-        query_dict = context['query_string']
         hints = driver_hints.Hints()
 
-        if query_dict is None:
+        if not request.params:
             return hints
 
-        for key in query_dict:
+        for key, value in request.params.items():
             # Check if this is an exact filter
             if supported_filters is None or key in supported_filters:
-                hints.add_filter(key, query_dict[key])
+                hints.add_filter(key, value)
                 continue
 
             # Check if it is an inexact filter
@@ -668,7 +650,7 @@ class V3Controller(wsgi.Application):
                 if comparator.startswith('i'):
                     case_sensitive = False
                     comparator = comparator[1:]
-                hints.add_filter(base_key, query_dict[key],
+                hints.add_filter(base_key, value,
                                  comparator=comparator,
                                  case_sensitive=case_sensitive)
 
@@ -678,39 +660,17 @@ class V3Controller(wsgi.Application):
         return hints
 
     def _require_matching_id(self, value, ref):
-        """Ensures the value matches the reference's ID, if any."""
+        """Ensure the value matches the reference's ID, if any."""
         if 'id' in ref and ref['id'] != value:
             raise exception.ValidationError('Cannot change ID')
 
-    def _require_matching_domain_id(self, ref_id, ref, get_member):
-        """Ensure the current domain ID matches the reference one, if any.
-
-        Provided we want domain IDs to be immutable, check whether any
-        domain_id specified in the ref dictionary matches the existing
-        domain_id for this entity.
-
-        :param ref_id: the ID of the entity
-        :param ref: the dictionary of new values proposed for this entity
-        :param get_member: The member function to call to get the current
-                           entity
-        :raises: :class:`keystone.exception.ValidationError`
-
-        """
-        # TODO(henry-nash): It might be safer and more efficient to do this
-        # check in the managers affected, so look to migrate this check to
-        # there in the future.
-        if CONF.domain_id_immutable and 'domain_id' in ref:
-            existing_ref = get_member(ref_id)
-            if ref['domain_id'] != existing_ref['domain_id']:
-                raise exception.ValidationError(_('Cannot change Domain ID'))
-
     def _assign_unique_id(self, ref):
-        """Generates and assigns a unique identifier to a reference."""
+        """Generate and assigns a unique identifier to a reference."""
         ref = ref.copy()
         ref['id'] = uuid.uuid4().hex
         return ref
 
-    def _get_domain_id_for_list_request(self, context):
+    def _get_domain_id_for_list_request(self, request):
         """Get the domain_id for a v3 list call.
 
         If we running with multiple domain drivers, then the caller must
@@ -721,21 +681,22 @@ class V3Controller(wsgi.Application):
             # We don't need to specify a domain ID in this case
             return
 
-        if context['query_string'].get('domain_id') is not None:
-            return context['query_string'].get('domain_id')
+        domain_id = request.params.get('domain_id')
+        if domain_id:
+            return domain_id
 
-        token_ref = utils.get_token_ref(context)
+        token_ref = utils.get_token_ref(request.context_dict)
 
         if token_ref.domain_scoped:
             return token_ref.domain_id
         elif token_ref.project_scoped:
             return token_ref.project_domain_id
         else:
-            LOG.warning(
-                _LW('No domain information specified as part of list request'))
-            raise exception.Unauthorized()
+            msg = _('No domain information specified as part of list request')
+            LOG.warning(msg)
+            raise exception.Unauthorized(msg)
 
-    def _get_domain_id_from_token(self, context):
+    def _get_domain_id_from_token(self, request):
         """Get the domain_id for a v3 create call.
 
         In the case of a v3 create entity call that does not specify a domain
@@ -743,42 +704,40 @@ class V3Controller(wsgi.Application):
         being used.
 
         """
-        try:
-            token_ref = utils.get_token_ref(context)
-        except exception.Unauthorized:
-            if context.get('is_admin'):
-                raise exception.ValidationError(
-                    _('You have tried to create a resource using the admin '
-                      'token. As this token is not within a domain you must '
-                      'explicitly include a domain for this resource to '
-                      'belong to.'))
-            raise
+        # return if domain scoped
+        if request.context.domain_id:
+            return request.context.domain_id
 
-        if token_ref.domain_scoped:
-            return token_ref.domain_id
-        else:
-            # TODO(henry-nash): We should issue an exception here since if
-            # a v3 call does not explicitly specify the domain_id in the
-            # entity, it should be using a domain scoped token.  However,
-            # the current tempest heat tests issue a v3 call without this.
-            # This is raised as bug #1283539.  Once this is fixed, we
-            # should remove the line below and replace it with an error.
-            #
-            # Ahead of actually changing the code to raise an exception, we
-            # issue a deprecation warning.
-            versionutils.report_deprecated_feature(
-                LOG,
-                _LW('Not specifying a domain during a create user, group or '
-                    'project call, and relying on falling back to the '
-                    'default domain, is deprecated as of Liberty and will be '
-                    'removed in the N release. Specify the domain explicitly '
-                    'or use a domain-scoped token'))
-            return CONF.identity.default_domain_id
+        if request.context.is_admin:
+            raise exception.ValidationError(
+                _('You have tried to create a resource using the admin '
+                  'token. As this token is not within a domain you must '
+                  'explicitly include a domain for this resource to '
+                  'belong to.'))
 
-    def _normalize_domain_id(self, context, ref):
+        # TODO(henry-nash): We should issue an exception here since if
+        # a v3 call does not explicitly specify the domain_id in the
+        # entity, it should be using a domain scoped token.  However,
+        # the current tempest heat tests issue a v3 call without this.
+        # This is raised as bug #1283539.  Once this is fixed, we
+        # should remove the line below and replace it with an error.
+        #
+        # Ahead of actually changing the code to raise an exception, we
+        # issue a deprecation warning.
+        versionutils.report_deprecated_feature(
+            LOG,
+            _LW('Not specifying a domain during a create user, group or '
+                'project call, and relying on falling back to the '
+                'default domain, is deprecated as of Liberty. There is no '
+                'plan to remove this compatibility, however, future API '
+                'versions may remove this, so please specify the domain '
+                'explicitly or use a domain-scoped token.'))
+        return CONF.identity.default_domain_id
+
+    def _normalize_domain_id(self, request, ref):
         """Fill in domain_id if not specified in a v3 call."""
         if not ref.get('domain_id'):
-            ref['domain_id'] = self._get_domain_id_from_token(context)
+            ref['domain_id'] = self._get_domain_id_from_token(request)
         return ref
 
     @staticmethod
@@ -786,7 +745,7 @@ class V3Controller(wsgi.Application):
         """Override v2 filter to let domain_id out for v3 calls."""
         return ref
 
-    def check_protection(self, context, prep_info, target_attr=None):
+    def check_protection(self, request, prep_info, target_attr=None):
         """Provide call protection for complex target attributes.
 
         As well as including the standard parameters from the original API
@@ -795,13 +754,13 @@ class V3Controller(wsgi.Application):
         they can be referenced by policy rules.
 
         """
-        if 'is_admin' in context and context['is_admin']:
+        if request.context.is_admin:
             LOG.warning(_LW('RBAC: Bypassing authorization'))
         else:
             action = 'identity:%s' % prep_info['f_name']
             # TODO(henry-nash) need to log the target attributes as well
             creds = _build_policy_check_credentials(self, action,
-                                                    context,
+                                                    request.context_dict,
                                                     prep_info['input_attr'])
             # Build the dict the policy engine will check against from both the
             # parameters passed into the call we are protecting (which was
@@ -833,3 +792,11 @@ class V3Controller(wsgi.Application):
         for blocked_param in blocked_keys:
             del ref[blocked_param]
         return ref
+
+    def _token_validation_window(self, request):
+        # NOTE(jamielennox): it's dumb that i have to put this here. We should
+        # only validate subject token in one place.
+        allow_expired = request.params.get('allow_expired')
+        allow_expired = strutils.bool_from_string(allow_expired, default=False)
+
+        return CONF.token.allow_expired_window if allow_expired else 0
