@@ -19,11 +19,11 @@
 """Utility methods for working with WSGI servers."""
 
 import copy
+import functools
 import itertools
 import re
 import wsgiref.util
 
-from oslo_config import cfg
 import oslo_i18n
 from oslo_log import log
 from oslo_serialization import jsonutils
@@ -31,20 +31,21 @@ from oslo_utils import importutils
 from oslo_utils import strutils
 import routes.middleware
 import six
+from six.moves import http_client
 import webob.dec
 import webob.exc
 
 from keystone.common import dependency
 from keystone.common import json_home
+from keystone.common import request as request_mod
 from keystone.common import utils
+import keystone.conf
 from keystone import exception
-from keystone.i18n import _
-from keystone.i18n import _LI
-from keystone.i18n import _LW
+from keystone.i18n import _, _LI, _LW
 from keystone.models import token_model
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 LOG = log.getLogger(__name__)
 
 # Environment variable used to pass the request context
@@ -77,43 +78,50 @@ def validate_token_bind(context, token_ref):
             # no bind provided and none required
             return
         else:
-            LOG.info(_LI("No bind information present in token"))
-            raise exception.Unauthorized()
+            msg = _('No bind information present in token.')
+            LOG.info(msg)
+            raise exception.Unauthorized(msg)
 
     # get the named mode if bind_mode is not one of the known
     name = None if permissive or bind_mode == 'required' else bind_mode
 
     if name and name not in bind:
-        LOG.info(_LI("Named bind mode %s not in bind information"), name)
-        raise exception.Unauthorized()
+        msg = (_('Named bind mode %(name)s not in bind information') %
+               {'name': name})
+        LOG.info(msg)
+        raise exception.Unauthorized(msg)
 
     for bind_type, identifier in bind.items():
         if bind_type == 'kerberos':
-            if not (context['environment'].get('AUTH_TYPE', '').lower()
-                    == 'negotiate'):
-                LOG.info(_LI("Kerberos credentials required and not present"))
-                raise exception.Unauthorized()
+            if (context['environment'].get('AUTH_TYPE', '').lower() !=
+                    'negotiate'):
+                msg = _('Kerberos credentials required and not present')
+                LOG.info(msg)
+                raise exception.Unauthorized(msg)
 
-            if not context['environment'].get('REMOTE_USER') == identifier:
-                LOG.info(_LI("Kerberos credentials do not match "
-                             "those in bind"))
-                raise exception.Unauthorized()
+            if context['environment'].get('REMOTE_USER') != identifier:
+                msg = _('Kerberos credentials do not match those in bind')
+                LOG.info(msg)
+                raise exception.Unauthorized(msg)
 
-            LOG.info(_LI("Kerberos bind authentication successful"))
+            LOG.info(_LI('Kerberos bind authentication successful'))
 
         elif bind_mode == 'permissive':
-            LOG.debug(("Ignoring unknown bind for permissive mode: "
-                       "{%(bind_type)s: %(identifier)s}"),
-                      {'bind_type': bind_type, 'identifier': identifier})
+            LOG.debug(("Ignoring unknown bind (due to permissive mode): "
+                       "{%(bind_type)s: %(identifier)s}"), {
+                           'bind_type': bind_type,
+                           'identifier': identifier})
         else:
-            LOG.info(_LI("Couldn't verify unknown bind: "
-                         "{%(bind_type)s: %(identifier)s}"),
-                     {'bind_type': bind_type, 'identifier': identifier})
-            raise exception.Unauthorized()
+            msg = _('Could not verify unknown bind: {%(bind_type)s: '
+                    '%(identifier)s}') % {
+                        'bind_type': bind_type,
+                        'identifier': identifier}
+            LOG.info(msg)
+            raise exception.Unauthorized(msg)
 
 
 def best_match_language(req):
-    """Determines the best available locale.
+    """Determine the best available locale.
 
     This returns best available locale based on the Accept-Language HTTP
     header passed in the request.
@@ -153,7 +161,9 @@ class BaseApplication(object):
         return cls(**local_config)
 
     def __call__(self, environ, start_response):
-        r"""Subclasses will probably want to implement __call__ like this:
+        r"""Provide subclasses on how to implement __call__.
+
+        Probably like this:
 
         @webob.dec.wsgify()
         def __call__(self, req):
@@ -191,47 +201,16 @@ class BaseApplication(object):
 
 @dependency.requires('assignment_api', 'policy_api', 'token_provider_api')
 class Application(BaseApplication):
-    @webob.dec.wsgify()
+
+    @webob.dec.wsgify(RequestClass=request_mod.Request)
     def __call__(self, req):
         arg_dict = req.environ['wsgiorg.routing_args'][1]
         action = arg_dict.pop('action')
         del arg_dict['controller']
 
-        # allow middleware up the stack to provide context, params and headers.
-        context = req.environ.get(CONTEXT_ENV, {})
-
-        try:
-            context['query_string'] = dict(req.params.items())
-        except UnicodeDecodeError as e:
-            # The webob package throws UnicodeError when a request cannot be
-            # decoded. Raise ValidationError instead to avoid an UnknownError.
-            msg = _('Query string is not UTF-8 encoded')
-            raise exception.ValidationError(msg)
-
-        context['headers'] = dict(req.headers.items())
-        context['path'] = req.environ['PATH_INFO']
-        scheme = req.environ.get(CONF.secure_proxy_ssl_header)
-        if scheme:
-            # NOTE(andrey-mp): "wsgi.url_scheme" contains the protocol used
-            # before the proxy removed it ('https' usually). So if
-            # the webob.Request instance is modified in order to use this
-            # scheme instead of the one defined by API, the call to
-            # webob.Request.relative_url() will return a URL with the correct
-            # scheme.
-            req.environ['wsgi.url_scheme'] = scheme
-        context['host_url'] = req.host_url
         params = req.environ.get(PARAMS_ENV, {})
-        # authentication and authorization attributes are set as environment
-        # values by the container and processed by the pipeline. The complete
-        # set is not yet known.
-        context['environment'] = req.environ
-        context['accept_header'] = req.accept
-        req.environ = None
-
         params.update(arg_dict)
 
-        context.setdefault('is_admin', False)
-
         # TODO(termie): do some basic normalization on methods
         method = getattr(self, action)
 
@@ -239,38 +218,42 @@ class Application(BaseApplication):
         # response code between GET and HEAD requests. The HTTP status should
         # be the same.
         LOG.info('%(req_method)s %(uri)s', {
-            'req_method': req.environ['REQUEST_METHOD'].upper(),
+            'req_method': req.method.upper(),
             'uri': wsgiref.util.request_uri(req.environ),
         })
 
         params = self._normalize_dict(params)
 
         try:
-            result = method(context, **params)
+            result = method(req, **params)
         except exception.Unauthorized as e:
             LOG.warning(
                 _LW("Authorization failed. %(exception)s from "
                     "%(remote_addr)s"),
                 {'exception': e, 'remote_addr': req.environ['REMOTE_ADDR']})
-            return render_exception(e, context=context,
+            return render_exception(e,
+                                    context=req.context_dict,
                                     user_locale=best_match_language(req))
         except exception.Error as e:
             LOG.warning(six.text_type(e))
-            return render_exception(e, context=context,
+            return render_exception(e,
+                                    context=req.context_dict,
                                     user_locale=best_match_language(req))
         except TypeError as e:
             LOG.exception(six.text_type(e))
             return render_exception(exception.ValidationError(e),
-                                    context=context,
+                                    context=req.context_dict,
                                     user_locale=best_match_language(req))
         except Exception as e:
             LOG.exception(six.text_type(e))
             return render_exception(exception.UnexpectedError(exception=e),
-                                    context=context,
+                                    context=req.context_dict,
                                     user_locale=best_match_language(req))
 
         if result is None:
-            return render_response(status=(204, 'No Content'))
+            return render_response(
+                status=(http_client.NO_CONTENT,
+                        http_client.responses[http_client.NO_CONTENT]))
         elif isinstance(result, six.string_types):
             return result
         elif isinstance(result, webob.Response):
@@ -279,15 +262,17 @@ class Application(BaseApplication):
             return result
 
         response_code = self._get_response_code(req)
-        return render_response(body=result, status=response_code,
-                               method=req.environ['REQUEST_METHOD'])
+        return render_response(body=result,
+                               status=response_code,
+                               method=req.method)
 
     def _get_response_code(self, req):
         req_method = req.environ['REQUEST_METHOD']
         controller = importutils.import_class('keystone.common.controller')
         code = None
         if isinstance(self, controller.V3Controller) and req_method == 'POST':
-            code = (201, 'Created')
+            code = (http_client.CREATED,
+                    http_client.responses[http_client.CREATED])
         return code
 
     def _normalize_arg(self, arg):
@@ -296,7 +281,7 @@ class Application(BaseApplication):
     def _normalize_dict(self, d):
         return {self._normalize_arg(k): v for (k, v) in d.items()}
 
-    def assert_admin(self, context):
+    def assert_admin(self, request):
         """Ensure the user is an admin.
 
         :raises keystone.exception.Unauthorized: if a token could not be
@@ -306,23 +291,24 @@ class Application(BaseApplication):
             does not have the admin role
 
         """
-        if not context['is_admin']:
-            user_token_ref = utils.get_token_ref(context)
+        request.assert_authenticated()
+
+        if not request.context.is_admin:
+            user_token_ref = utils.get_token_ref(request.context_dict)
 
-            validate_token_bind(context, user_token_ref)
             creds = copy.deepcopy(user_token_ref.metadata)
 
             try:
                 creds['user_id'] = user_token_ref.user_id
             except exception.UnexpectedError:
                 LOG.debug('Invalid user')
-                raise exception.Unauthorized()
+                raise exception.Unauthorized(_('Invalid user'))
 
             if user_token_ref.project_scoped:
                 creds['tenant_id'] = user_token_ref.project_id
             else:
                 LOG.debug('Invalid tenant')
-                raise exception.Unauthorized()
+                raise exception.Unauthorized(_('Invalid tenant'))
 
             creds['roles'] = user_token_ref.role_names
             # Accept either is_admin or the admin role
@@ -333,7 +319,7 @@ class Application(BaseApplication):
         return ref.get(attribute) is None or ref.get(attribute) == ''
 
     def _require_attribute(self, ref, attribute):
-        """Ensures the reference contains the specified attribute.
+        """Ensure the reference contains the specified attribute.
 
         Raise a ValidationError if the given attribute is not present
         """
@@ -342,7 +328,7 @@ class Application(BaseApplication):
             raise exception.ValidationError(message=msg)
 
     def _require_attributes(self, ref, attrs):
-        """Ensures the reference contains the specified attributes.
+        """Ensure the reference contains the specified attributes.
 
         Raise a ValidationError if any of the given attributes is not present
         """
@@ -353,20 +339,6 @@ class Application(BaseApplication):
             msg = _('%s field(s) cannot be empty') % ', '.join(missing_attrs)
             raise exception.ValidationError(message=msg)
 
-    def _get_trust_id_for_request(self, context):
-        """Get the trust_id for a call.
-
-        Retrieve the trust_id from the token
-        Returns None if token is not trust scoped
-        """
-        if ('token_id' not in context or
-                context.get('token_id') == CONF.admin_token):
-            LOG.debug(('will not lookup trust as the request auth token is '
-                       'either absent or it is the system admin token'))
-            return None
-        token_ref = utils.get_token_ref(context)
-        return token_ref.trust_id
-
     @classmethod
     def base_url(cls, context, endpoint_type):
         url = CONF['%s_endpoint' % endpoint_type]
@@ -393,6 +365,30 @@ class Application(BaseApplication):
         return url.rstrip('/')
 
 
+def middleware_exceptions(method):
+
+    @functools.wraps(method)
+    def _inner(self, request):
+        try:
+            return method(self, request)
+        except exception.Error as e:
+            LOG.warning(six.text_type(e))
+            return render_exception(e, request=request,
+                                    user_locale=best_match_language(request))
+        except TypeError as e:
+            LOG.exception(six.text_type(e))
+            return render_exception(exception.ValidationError(e),
+                                    request=request,
+                                    user_locale=best_match_language(request))
+        except Exception as e:
+            LOG.exception(six.text_type(e))
+            return render_exception(exception.UnexpectedError(exception=e),
+                                    request=request,
+                                    user_locale=best_match_language(request))
+
+    return _inner
+
+
 class Middleware(Application):
     """Base WSGI middleware.
 
@@ -428,28 +424,14 @@ class Middleware(Application):
         """Do whatever you'd like to the response, based on the request."""
         return response
 
-    @webob.dec.wsgify()
+    @webob.dec.wsgify(RequestClass=request_mod.Request)
+    @middleware_exceptions
     def __call__(self, request):
-        try:
-            response = self.process_request(request)
-            if response:
-                return response
-            response = request.get_response(self.application)
-            return self.process_response(request, response)
-        except exception.Error as e:
-            LOG.warning(six.text_type(e))
-            return render_exception(e, request=request,
-                                    user_locale=best_match_language(request))
-        except TypeError as e:
-            LOG.exception(six.text_type(e))
-            return render_exception(exception.ValidationError(e),
-                                    request=request,
-                                    user_locale=best_match_language(request))
-        except Exception as e:
-            LOG.exception(six.text_type(e))
-            return render_exception(exception.UnexpectedError(exception=e),
-                                    request=request,
-                                    user_locale=best_match_language(request))
+        response = self.process_request(request)
+        if response:
+            return response
+        response = request.get_response(self.application)
+        return self.process_response(request, response)
 
 
 class Debug(Middleware):
@@ -460,7 +442,7 @@ class Debug(Middleware):
 
     """
 
-    @webob.dec.wsgify()
+    @webob.dec.wsgify(RequestClass=request_mod.Request)
     def __call__(self, req):
         if not hasattr(LOG, 'isEnabledFor') or LOG.isEnabledFor(LOG.debug):
             LOG.debug('%s %s %s', ('*' * 20), 'REQUEST ENVIRON', ('*' * 20))
@@ -524,7 +506,7 @@ class Router(object):
         self._router = routes.middleware.RoutesMiddleware(self._dispatch,
                                                           self.map)
 
-    @webob.dec.wsgify()
+    @webob.dec.wsgify(RequestClass=request_mod.Request)
     def __call__(self, req):
         """Route the incoming request to a controller based on self.map.
 
@@ -534,7 +516,7 @@ class Router(object):
         return self._router
 
     @staticmethod
-    @webob.dec.wsgify()
+    @webob.dec.wsgify(RequestClass=request_mod.Request)
     def _dispatch(req):
         """Dispatch the request to the appropriate controller.
 
@@ -701,7 +683,7 @@ class V3ExtensionRouter(ExtensionRouter, RoutersBase):
     def _update_version_response(self, response_data):
         response_data['resources'].update(self.v3_resources)
 
-    @webob.dec.wsgify()
+    @webob.dec.wsgify(RequestClass=request_mod.Request)
     def __call__(self, request):
         if request.path_info != '/':
             # Not a request for version info so forward to super.
@@ -709,7 +691,7 @@ class V3ExtensionRouter(ExtensionRouter, RoutersBase):
 
         response = request.get_response(self.application)
 
-        if response.status_code != 200:
+        if response.status_code != http_client.OK:
             # The request failed, so don't update the response.
             return response
 
@@ -726,7 +708,7 @@ class V3ExtensionRouter(ExtensionRouter, RoutersBase):
 
 
 def render_response(body=None, status=None, headers=None, method=None):
-    """Forms a WSGI response."""
+    """Form a WSGI response."""
     if headers is None:
         headers = []
     else:
@@ -735,7 +717,8 @@ def render_response(body=None, status=None, headers=None, method=None):
 
     if body is None:
         body = b''
-        status = status or (204, 'No Content')
+        status = status or (http_client.NO_CONTENT,
+                            http_client.responses[http_client.NO_CONTENT])
     else:
         content_types = [v for h, v in headers if h == 'Content-Type']
         if content_types:
@@ -747,7 +730,8 @@ def render_response(body=None, status=None, headers=None, method=None):
             body = jsonutils.dump_as_bytes(body, cls=utils.SmarterEncoder)
             if content_type is None:
                 headers.append(('Content-Type', 'application/json'))
-        status = status or (200, 'OK')
+        status = status or (http_client.OK,
+                            http_client.responses[http_client.OK])
 
     # NOTE(davechen): `mod_wsgi` follows the standards from pep-3333 and
     # requires the value in response header to be binary type(str) on python2,
@@ -780,7 +764,7 @@ def render_response(body=None, status=None, headers=None, method=None):
     headers = _convert_to_str(headers)
 
     resp = webob.Response(body=body,
-                          status='%s %s' % status,
+                          status='%d %s' % status,
                           headerlist=headers)
 
     if method and method.upper() == 'HEAD':
@@ -801,7 +785,7 @@ def render_response(body=None, status=None, headers=None, method=None):
 
 
 def render_exception(error, context=None, request=None, user_locale=None):
-    """Forms a WSGI response based on the current error."""
+    """Form a WSGI response based on the current error."""
     error_message = error.args[0]
     message = oslo_i18n.translate(error_message, desired_locale=user_locale)
     if message is error_message:
@@ -819,7 +803,7 @@ def render_exception(error, context=None, request=None, user_locale=None):
         body['error']['identity'] = error.authentication
     elif isinstance(error, exception.Unauthorized):
         # NOTE(gyee): we only care about the request environment in the
-        # context. Also, its OK to pass the environemt as it is read-only in
+        # context. Also, its OK to pass the environment as it is read-only in
         # Application.base_url()
         local_context = {}
         if request:
