@@ -36,7 +36,6 @@ from keystone.conf import kvs
 from keystone.conf import ldap
 from keystone.conf import memcache
 from keystone.conf import oauth1
-from keystone.conf import os_inherit
 from keystone.conf import paste_deploy
 from keystone.conf import policy
 from keystone.conf import resource
@@ -72,7 +71,6 @@ conf_modules = [
     ldap,
     memcache,
     oauth1,
-    os_inherit,
     paste_deploy,
     policy,
     resource,
@@ -143,34 +141,32 @@ def configure(conf=None):
 
 def set_external_opts_defaults():
     """Update default configuration options for oslo.middleware."""
-    # CORS Defaults
-    # TODO(krotscheck): Update with https://review.openstack.org/#/c/285368/
-    cfg.set_defaults(cors.CORS_OPTS,
-                     allow_headers=['X-Auth-Token',
-                                    'X-Openstack-Request-Id',
-                                    'X-Subject-Token',
-                                    'X-Project-Id',
-                                    'X-Project-Name',
-                                    'X-Project-Domain-Id',
-                                    'X-Project-Domain-Name',
-                                    'X-Domain-Id',
-                                    'X-Domain-Name'],
-                     expose_headers=['X-Auth-Token',
-                                     'X-Openstack-Request-Id',
-                                     'X-Subject-Token'],
-                     allow_methods=['GET',
-                                    'PUT',
-                                    'POST',
-                                    'DELETE',
-                                    'PATCH']
-                     )
+    cors.set_defaults(
+        allow_headers=['X-Auth-Token',
+                       'X-Openstack-Request-Id',
+                       'X-Subject-Token',
+                       'X-Project-Id',
+                       'X-Project-Name',
+                       'X-Project-Domain-Id',
+                       'X-Project-Domain-Name',
+                       'X-Domain-Id',
+                       'X-Domain-Name'],
+        expose_headers=['X-Auth-Token',
+                        'X-Openstack-Request-Id',
+                        'X-Subject-Token'],
+        allow_methods=['GET',
+                       'PUT',
+                       'POST',
+                       'DELETE',
+                       'PATCH']
+    )
 
     # configure OSprofiler options
     profiler.set_defaults(CONF, enabled=False, trace_sqlalchemy=False)
 
     # Oslo.cache is always enabled by default for request-local caching
     # TODO(morganfainberg): Fix this to not use internal interface when
-    # oslo.cache has proper interface to set defaults added. This is is
+    # oslo.cache has proper interface to set defaults added. This is
     # just a bad way to do this.
     opts = cache._opts.list_opts()
     for opt_list in opts:
