@@ -10,13 +10,10 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import hashlib
 import sys
 
 from oslo_config import cfg
-from oslo_log import versionutils
 
-from keystone.conf import constants
 from keystone.conf import utils
 
 
@@ -62,18 +59,16 @@ potential security impact of a compromised token.
 
 provider = cfg.StrOpt(
     'provider',
-    default='uuid',
+    default='fernet',
     help=utils.fmt("""
 Entry point for the token provider in the `keystone.token.provider` namespace.
 The token provider controls the token construction, validation, and revocation
-operations. Keystone includes `fernet`, `pkiz`, `pki`, and `uuid` token
+operations. Keystone includes `fernet` and `uuid` token
 providers. `uuid` tokens must be persisted (using the backend specified in the
 `[token] driver` option), but do not require any extra configuration or setup.
 `fernet` tokens do not need to be persisted at all, but require that you run
 `keystone-manage fernet_setup` (also see the `keystone-manage fernet_rotate`
-command). `pki` and `pkiz` tokens can be validated offline, without making HTTP
-calls to keystone, but require that certificates be installed and distributed
-to facilitate signing tokens and later validating those signatures.
+command).
 """))
 
 driver = cfg.StrOpt(
@@ -81,13 +76,11 @@ driver = cfg.StrOpt(
     default='sql',
     help=utils.fmt("""
 Entry point for the token persistence backend driver in the
-`keystone.token.persistence` namespace. Keystone provides `kvs`, `memcache`,
-`memcache_pool`, and `sql` drivers. The `kvs` backend depends on the
-configuration in the `[kvs]` section. The `memcache` and `memcache_pool`
-options depend on the configuration in the `[memcache]` section. The `sql`
-option (default) depends on the options in your `[database]` section. If you're
-using the `fernet` `[token] provider`, this backend will not be utilized to
-persist tokens at all.
+`keystone.token.persistence` namespace. Keystone provides `kvs` and `sql`
+drivers. The `kvs` backend depends on the configuration in the `[kvs]` section.
+The `sql` option (default) depends on the options in your `[database]` section.
+If you're using the `fernet` `[token] provider`, this backend will not be
+utilized to persist tokens at all.
 """))
 
 caching = cfg.BoolOpt(
@@ -130,26 +123,6 @@ for tokens with a more specific scope) or to provide their credentials in every
 request for a scoped token to avoid re-scoping altogether.
 """))
 
-# This attribute only exists in Python 2.7.8+ or 3.2+
-hash_choices = getattr(hashlib, 'algorithms_guaranteed', None)
-hash_choices = sorted(hash_choices) if hash_choices else None
-hash_algorithm = cfg.StrOpt(
-    'hash_algorithm',
-    default='md5',
-    choices=hash_choices,
-    deprecated_for_removal=True,
-    deprecated_reason=constants._DEPRECATE_PKI_MSG,
-    deprecated_since=versionutils.deprecated.MITAKA,
-    help=utils.fmt("""
-This controls the hash algorithm to use to uniquely identify PKI tokens without
-having to transmit the entire token to keystone (which may be several
-kilobytes). This can be set to any algorithm that hashlib supports. WARNING:
-Before changing this value, the `auth_token` middleware protecting all other
-services must be configured with the set of hash algorithms to expect from
-keystone (both your old and new value for this option), otherwise token
-revocation will not be processed correctly.
-"""))
-
 infer_roles = cfg.BoolOpt(
     'infer_roles',
     default=True,
@@ -161,12 +134,23 @@ other role assignments.
 
 cache_on_issue = cfg.BoolOpt(
     'cache_on_issue',
-    default=False,
+    default=True,
     help=utils.fmt("""
 Enable storing issued token data to token validation cache so that first token
-validation doesn't actually cause full validation cycle.
+validation doesn't actually cause full validation cycle. This option has no
+effect unless global caching and token caching are enabled.
 """))
 
+allow_expired_window = cfg.IntOpt(
+    'allow_expired_window',
+    default=48 * 60 * 60,
+    help=utils.fmt("""
+This controls the number of seconds that a token can be retrieved for beyond
+the built-in expiry time. This allows long running operations to succeed.
+Defaults to two days.
+"""))
+
+
 GROUP_NAME = __name__.split('.')[-1]
 ALL_OPTS = [
     bind,
@@ -178,9 +162,9 @@ ALL_OPTS = [
     cache_time,
     revoke_by_id,
     allow_rescope_scoped_token,
-    hash_algorithm,
     infer_roles,
     cache_on_issue,
+    allow_expired_window,
 ]
 
 
