@@ -17,7 +17,7 @@
 This service allows the creation of access/secret credentials used for
 the ec2 interop layer of OpenStack.
 
-A user can create as many access/secret pairs, each of which map to a
+A user can create as many access/secret pairs, each of which is mapped to a
 specific project.  This is required because OpenStack supports a user
 belonging to multiple projects, whereas the signatures created on ec2-style
 requests don't allow specification of which project the user wishes to act
@@ -39,6 +39,7 @@ import uuid
 from keystoneclient.contrib.ec2 import utils as ec2_utils
 from oslo_serialization import jsonutils
 import six
+from six.moves import http_client
 
 from keystone.common import controller
 from keystone.common import dependency
@@ -46,6 +47,9 @@ from keystone.common import utils
 from keystone.common import wsgi
 from keystone import exception
 from keystone.i18n import _
+from keystone.token import controllers as token_controllers
+
+CRED_TYPE_EC2 = 'ec2'
 
 
 @dependency.requires('assignment_api', 'catalog_api', 'credential_api',
@@ -75,13 +79,14 @@ class Ec2ControllerCommon(object):
                                         signature):
                     return True
                 raise exception.Unauthorized(
-                    message='Invalid EC2 signature.')
+                    message=_('Invalid EC2 signature.'))
             else:
                 raise exception.Unauthorized(
-                    message='EC2 signature not supplied.')
+                    message=_('EC2 signature not supplied.'))
         # Raise the exception when credentials.get('signature') is None
         else:
-            raise exception.Unauthorized(message='EC2 signature not supplied.')
+            raise exception.Unauthorized(
+                message=_('EC2 signature not supplied.'))
 
     @abc.abstractmethod
     def authenticate(self, context, credentials=None, ec2Credentials=None):
@@ -109,9 +114,8 @@ class Ec2ControllerCommon(object):
     def _authenticate(self, credentials=None, ec2credentials=None):
         """Common code shared between the V2 and V3 authenticate methods.
 
-        :returns: user_ref, tenant_ref, metadata_ref, roles_ref, catalog_ref
+        :returns: user_ref, tenant_ref, roles_ref, catalog_ref
         """
-
         # FIXME(ja): validate that a service token was used!
 
         # NOTE(termie): backwards compat hack
@@ -119,7 +123,8 @@ class Ec2ControllerCommon(object):
             credentials = ec2credentials
 
         if 'access' not in credentials:
-            raise exception.Unauthorized(message='EC2 signature not supplied.')
+            raise exception.Unauthorized(
+                message=_('EC2 signature not supplied.'))
 
         creds_ref = self._get_credentials(credentials['access'])
         self.check_signature(creds_ref, credentials)
@@ -128,15 +133,6 @@ class Ec2ControllerCommon(object):
         # TODO(termie): this is copied from TokenController.authenticate
         tenant_ref = self.resource_api.get_project(creds_ref['tenant_id'])
         user_ref = self.identity_api.get_user(creds_ref['user_id'])
-        metadata_ref = {}
-        metadata_ref['roles'] = (
-            self.assignment_api.get_roles_for_user_and_project(
-                user_ref['id'], tenant_ref['id']))
-
-        trust_id = creds_ref.get('trust_id')
-        if trust_id:
-            metadata_ref['trust_id'] = trust_id
-            metadata_ref['trustee_user_id'] = user_ref['id']
 
         # Validate that the auth info is valid and nothing is disabled
         try:
@@ -150,40 +146,41 @@ class Ec2ControllerCommon(object):
             six.reraise(exception.Unauthorized, exception.Unauthorized(e),
                         sys.exc_info()[2])
 
-        roles = metadata_ref.get('roles', [])
+        roles = self.assignment_api.get_roles_for_user_and_project(
+            user_ref['id'], tenant_ref['id']
+        )
         if not roles:
-            raise exception.Unauthorized(message='User not valid for tenant.')
+            raise exception.Unauthorized(
+                message=_('User not valid for tenant.'))
         roles_ref = [self.role_api.get_role(role_id) for role_id in roles]
 
         catalog_ref = self.catalog_api.get_catalog(
             user_ref['id'], tenant_ref['id'])
 
-        return user_ref, tenant_ref, metadata_ref, roles_ref, catalog_ref
+        return user_ref, tenant_ref, roles_ref, catalog_ref
 
-    def create_credential(self, context, user_id, tenant_id):
+    def create_credential(self, request, user_id, tenant_id):
         """Create a secret/access pair for use with ec2 style auth.
 
         Generates a new set of credentials that map the user/tenant
         pair.
 
-        :param context: standard context
+        :param request: current request
         :param user_id: id of user
         :param tenant_id: id of tenant
         :returns: credential: dict of ec2 credential
         """
-
         self.identity_api.get_user(user_id)
         self.resource_api.get_project(tenant_id)
-        trust_id = self._get_trust_id_for_request(context)
         blob = {'access': uuid.uuid4().hex,
                 'secret': uuid.uuid4().hex,
-                'trust_id': trust_id}
+                'trust_id': request.context.trust_id}
         credential_id = utils.hash_access_key(blob['access'])
         cred_ref = {'user_id': user_id,
                     'project_id': tenant_id,
                     'blob': jsonutils.dumps(blob),
                     'id': credential_id,
-                    'type': 'ec2'}
+                    'type': CRED_TYPE_EC2}
         self.credential_api.create_credential(credential_id, cred_ref)
         return {'credential': self._convert_v3_to_ec2_credential(cred_ref)}
 
@@ -193,10 +190,9 @@ class Ec2ControllerCommon(object):
         :param user_id: id of user
         :returns: credentials: list of ec2 credential dicts
         """
-
         self.identity_api.get_user(user_id)
         credential_refs = self.credential_api.list_credentials_for_user(
-            user_id)
+            user_id, type=CRED_TYPE_EC2)
         return {'credentials':
                 [self._convert_v3_to_ec2_credential(credential)
                     for credential in credential_refs]}
@@ -210,7 +206,6 @@ class Ec2ControllerCommon(object):
         :param credential_id: access key for credentials
         :returns: credential: dict of ec2 credential
         """
-
         self.identity_api.get_user(user_id)
         return {'credential': self._get_credentials(credential_id)}
 
@@ -223,7 +218,6 @@ class Ec2ControllerCommon(object):
         :param credential_id: access key for credentials
         :returns: bool: success
         """
-
         self.identity_api.get_user(user_id)
         self._get_credentials(credential_id)
         ec2_credential_id = utils.hash_access_key(credential_id)
@@ -249,63 +243,74 @@ class Ec2ControllerCommon(object):
         """Return credentials from an ID.
 
         :param credential_id: id of credential
-        :raises exception.Unauthorized: when credential id is invalid
+        :raises keystone.exception.Unauthorized: when credential id is invalid
+            or when the credential type is not ec2
         :returns: credential: dict of ec2 credential.
         """
         ec2_credential_id = utils.hash_access_key(credential_id)
-        creds = self.credential_api.get_credential(ec2_credential_id)
-        if not creds:
-            raise exception.Unauthorized(message='EC2 access key not found.')
-        return self._convert_v3_to_ec2_credential(creds)
+        cred = self.credential_api.get_credential(ec2_credential_id)
+        if not cred or cred['type'] != CRED_TYPE_EC2:
+            raise exception.Unauthorized(
+                message=_('EC2 access key not found.'))
+        return self._convert_v3_to_ec2_credential(cred)
+
+    def render_token_data_response(self, token_id, token_data):
+        """Render token data HTTP response.
+
+        Stash token ID into the X-Subject-Token header.
+
+        """
+        status = (http_client.OK,
+                  http_client.responses[http_client.OK])
+        headers = [('X-Subject-Token', token_id)]
+
+        return wsgi.render_response(body=token_data,
+                                    status=status,
+                                    headers=headers)
 
 
 @dependency.requires('policy_api', 'token_provider_api')
 class Ec2Controller(Ec2ControllerCommon, controller.V2Controller):
 
-    @controller.v2_deprecated
-    def authenticate(self, context, credentials=None, ec2Credentials=None):
-        (user_ref, tenant_ref, metadata_ref, roles_ref,
-         catalog_ref) = self._authenticate(credentials=credentials,
-                                           ec2credentials=ec2Credentials)
+    @controller.v2_ec2_deprecated
+    def authenticate(self, request, credentials=None, ec2Credentials=None):
+        (user_ref, project_ref, roles_ref, catalog_ref) = self._authenticate(
+            credentials=credentials, ec2credentials=ec2Credentials
+        )
 
-        # NOTE(morganfainberg): Make sure the data is in correct form since it
-        # might be consumed external to Keystone and this is a v2.0 controller.
-        # The token provider does not explicitly care about user_ref version
-        # in this case, but the data is stored in the token itself and should
-        # match the version
-        user_ref = self.v3_to_v2_user(user_ref)
-        auth_token_data = dict(user=user_ref,
-                               tenant=tenant_ref,
-                               metadata=metadata_ref,
-                               id='placeholder')
-        (token_id, token_data) = self.token_provider_api.issue_v2_token(
-            auth_token_data, roles_ref, catalog_ref)
+        method_names = ['ec2credential']
+
+        token_id, token_data = self.token_provider_api.issue_token(
+            user_ref['id'], method_names, project_id=project_ref['id'])
+
+        v2_helper = token_controllers.V2TokenDataHelper()
+        token_data = v2_helper.v3_to_v2_token(token_data, token_id)
         return token_data
 
-    @controller.v2_deprecated
-    def get_credential(self, context, user_id, credential_id):
-        if not self._is_admin(context):
-            self._assert_identity(context, user_id)
+    @controller.v2_ec2_deprecated
+    def get_credential(self, request, user_id, credential_id):
+        if not self._is_admin(request):
+            self._assert_identity(request.context_dict, user_id)
         return super(Ec2Controller, self).get_credential(user_id,
                                                          credential_id)
 
-    @controller.v2_deprecated
-    def get_credentials(self, context, user_id):
-        if not self._is_admin(context):
-            self._assert_identity(context, user_id)
+    @controller.v2_ec2_deprecated
+    def get_credentials(self, request, user_id):
+        if not self._is_admin(request):
+            self._assert_identity(request.context_dict, user_id)
         return super(Ec2Controller, self).get_credentials(user_id)
 
-    @controller.v2_deprecated
-    def create_credential(self, context, user_id, tenant_id):
-        if not self._is_admin(context):
-            self._assert_identity(context, user_id)
-        return super(Ec2Controller, self).create_credential(context, user_id,
-                                                            tenant_id)
+    @controller.v2_ec2_deprecated
+    def create_credential(self, request, user_id, tenant_id):
+        if not self._is_admin(request):
+            self._assert_identity(request.context_dict, user_id)
+        return super(Ec2Controller, self).create_credential(
+            request, user_id, tenant_id)
 
-    @controller.v2_deprecated
-    def delete_credential(self, context, user_id, credential_id):
-        if not self._is_admin(context):
-            self._assert_identity(context, user_id)
+    @controller.v2_ec2_deprecated
+    def delete_credential(self, request, user_id, credential_id):
+        if not self._is_admin(request):
+            self._assert_identity(request.context_dict, user_id)
             self._assert_owner(user_id, credential_id)
         return super(Ec2Controller, self).delete_credential(user_id,
                                                             credential_id)
@@ -315,7 +320,7 @@ class Ec2Controller(Ec2ControllerCommon, controller.V2Controller):
 
         :param context: standard context
         :param user_id: id of user
-        :raises exception.Forbidden: when token is invalid
+        :raises keystone.exception.Forbidden: when token is invalid
 
         """
         token_ref = utils.get_token_ref(context)
@@ -323,7 +328,7 @@ class Ec2Controller(Ec2ControllerCommon, controller.V2Controller):
         if token_ref.user_id != user_id:
             raise exception.Forbidden(_('Token belongs to another user'))
 
-    def _is_admin(self, context):
+    def _is_admin(self, request):
         """Wrap admin assertion error return statement.
 
         :param context: standard context
@@ -333,7 +338,7 @@ class Ec2Controller(Ec2ControllerCommon, controller.V2Controller):
         try:
             # NOTE(morganfainberg): policy_api is required for assert_admin
             # to properly perform policy enforcement.
-            self.assert_admin(context)
+            self.assert_admin(request)
             return True
         except (exception.Forbidden, exception.Unauthorized):
             return False
@@ -343,7 +348,7 @@ class Ec2Controller(Ec2ControllerCommon, controller.V2Controller):
 
         :param user_id: expected credential owner
         :param credential_id: id of credential object
-        :raises exception.Forbidden: on failure
+        :raises keystone.exception.Forbidden: on failure
 
         """
         ec2_credential_id = utils.hash_access_key(credential_id)
@@ -358,10 +363,7 @@ class Ec2ControllerV3(Ec2ControllerCommon, controller.V3Controller):
     collection_name = 'credentials'
     member_name = 'credential'
 
-    def __init__(self):
-        super(Ec2ControllerV3, self).__init__()
-
-    def _check_credential_owner_and_user_id_match(self, context, prep_info,
+    def _check_credential_owner_and_user_id_match(self, request, prep_info,
                                                   user_id, credential_id):
         # NOTE(morganfainberg): this method needs to capture the arguments of
         # the method that is decorated with @controller.protected() (with
@@ -375,39 +377,41 @@ class Ec2ControllerV3(Ec2ControllerCommon, controller.V3Controller):
         ref['credential'] = self.credential_api.get_credential(credential_id)
         # NOTE(morganfainberg): policy_api is required for this
         # check_protection to properly be able to perform policy enforcement.
-        self.check_protection(context, prep_info, ref)
+        self.check_protection(request, prep_info, ref)
 
     def authenticate(self, context, credentials=None, ec2Credentials=None):
-        (user_ref, project_ref, metadata_ref, roles_ref,
-         catalog_ref) = self._authenticate(credentials=credentials,
-                                           ec2credentials=ec2Credentials)
+        (user_ref, project_ref, roles_ref, catalog_ref) = self._authenticate(
+            credentials=credentials, ec2credentials=ec2Credentials
+        )
 
         method_names = ['ec2credential']
 
-        token_id, token_data = self.token_provider_api.issue_v3_token(
-            user_ref['id'], method_names, project_id=project_ref['id'],
-            metadata_ref=metadata_ref)
-        return render_token_data_response(token_id, token_data)
+        token_id, token_data = self.token_provider_api.issue_token(
+            user_ref['id'], method_names, project_id=project_ref['id'])
+        return self.render_token_data_response(token_id, token_data)
 
     @controller.protected(callback=_check_credential_owner_and_user_id_match)
-    def ec2_get_credential(self, context, user_id, credential_id):
+    def ec2_get_credential(self, request, user_id, credential_id):
         ref = super(Ec2ControllerV3, self).get_credential(user_id,
                                                           credential_id)
-        return Ec2ControllerV3.wrap_member(context, ref['credential'])
+        return Ec2ControllerV3.wrap_member(request.context_dict,
+                                           ref['credential'])
 
     @controller.protected()
-    def ec2_list_credentials(self, context, user_id):
+    def ec2_list_credentials(self, request, user_id):
         refs = super(Ec2ControllerV3, self).get_credentials(user_id)
-        return Ec2ControllerV3.wrap_collection(context, refs['credentials'])
+        return Ec2ControllerV3.wrap_collection(request.context_dict,
+                                               refs['credentials'])
 
     @controller.protected()
-    def ec2_create_credential(self, context, user_id, tenant_id):
-        ref = super(Ec2ControllerV3, self).create_credential(context, user_id,
-                                                             tenant_id)
-        return Ec2ControllerV3.wrap_member(context, ref['credential'])
+    def ec2_create_credential(self, request, user_id, tenant_id):
+        ref = super(Ec2ControllerV3, self).create_credential(
+            request, user_id, tenant_id)
+        return Ec2ControllerV3.wrap_member(request.context_dict,
+                                           ref['credential'])
 
     @controller.protected(callback=_check_credential_owner_and_user_id_match)
-    def ec2_delete_credential(self, context, user_id, credential_id):
+    def ec2_delete_credential(self, request, user_id, credential_id):
         return super(Ec2ControllerV3, self).delete_credential(user_id,
                                                               credential_id)
 
@@ -419,15 +423,3 @@ class Ec2ControllerV3(Ec2ControllerCommon, controller.V3Controller):
             'credential_id': ref['access']}
         ref.setdefault('links', {})
         ref['links']['self'] = url
-
-
-def render_token_data_response(token_id, token_data):
-    """Render token data HTTP response.
-
-    Stash token ID into the X-Subject-Token header.
-
-    """
-    headers = [('X-Subject-Token', token_id)]
-
-    return wsgi.render_response(body=token_data,
-                                status=(200, 'OK'), headers=headers)
