@@ -447,13 +447,8 @@ class DomainV3(controller.V3Controller):
         :returns: list of accessible domains
 
         """
-        domains = self.assignment_api.list_domains_for_groups(
-            request.auth_context['group_ids'])
-        domains = domains + self.assignment_api.list_domains_for_user(
-            request.auth_context['user_id'])
-        # remove duplicates
-        domains = [dict(t) for t in set([tuple(d.items()) for d in domains])]
-        return DomainV3.wrap_collection(request.context_dict, domains)
+        controller = auth_controllers.Auth()
+        return controller.get_auth_domains(request)
 
 
 @dependency.requires('assignment_api', 'resource_api')
@@ -473,14 +468,8 @@ class ProjectAssignmentV3(controller.V3Controller):
         :returns: list of accessible projects
 
         """
-        projects = self.assignment_api.list_projects_for_groups(
-            request.auth_context['group_ids'])
-        projects = projects + self.assignment_api.list_projects_for_user(
-            request.auth_context['user_id'])
-        # remove duplicates
-        projects = [dict(t) for t in set([tuple(d.items()) for d in projects])]
-        return ProjectAssignmentV3.wrap_collection(request.context_dict,
-                                                   projects)
+        controller = auth_controllers.Auth()
+        return controller.get_auth_projects(request)
 
 
 @dependency.requires('federation_api')
