@@ -1,4 +1,4 @@
-# Copyright (c) 2011 OpenStack, LLC.
+# Copyright (c) 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,17 +13,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""Policy engine for keystone"""
+"""Policy engine for keystone."""
 
-from oslo_config import cfg
 from oslo_log import log
 from oslo_policy import policy as common_policy
 
+import keystone.conf
 from keystone import exception
-from keystone import policy
+from keystone.policy.backends import base
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 LOG = log.getLogger(__name__)
 
 
@@ -42,7 +42,7 @@ def init():
 
 
 def enforce(credentials, action, target, do_raise=True):
-    """Verifies that the action is valid on the target in this context.
+    """Verify that the action is valid on the target in this context.
 
     :param credentials: user credentials
     :param action: string representing the action to be checked, which should
@@ -69,9 +69,10 @@ def enforce(credentials, action, target, do_raise=True):
     return _ENFORCER.enforce(action, target, credentials, **extra)
 
 
-class Policy(policy.PolicyDriverV8):
+class Policy(base.PolicyDriverBase):
     def enforce(self, credentials, action, target):
-        LOG.debug('enforce %(action)s: %(credentials)s', {
+        msg = 'enforce %(action)s: %(credentials)s'
+        LOG.debug(msg, {
             'action': action,
             'credentials': credentials})
         enforce(credentials, action, target)
