@@ -12,8 +12,6 @@
 
 """Main entry point into the Revoke service."""
 
-from oslo_log import versionutils
-
 from keystone.common import cache
 from keystone.common import dependency
 from keystone.common import extension
@@ -23,7 +21,6 @@ from keystone import exception
 from keystone.i18n import _
 from keystone.models import revoke_model
 from keystone import notifications
-from keystone.revoke.backends import base
 
 
 CONF = keystone.conf.CONF
@@ -31,7 +28,7 @@ CONF = keystone.conf.CONF
 
 EXTENSION_DATA = {
     'name': 'OpenStack Revoke API',
-    'namespace': 'http://docs.openstack.org/identity/api/ext/'
+    'namespace': 'https://docs.openstack.org/identity/api/ext/'
                  'OS-REVOKE/v1.0',
     'alias': 'OS-REVOKE',
     'updated': '2014-02-24T20:51:0-00:00',
@@ -40,7 +37,7 @@ EXTENSION_DATA = {
         {
             'rel': 'describedby',
             'type': 'text/html',
-            'href': 'http://developer.openstack.org/'
+            'href': 'https://developer.openstack.org/'
                     'api-ref-identity-v3-ext.html',
         }
     ]}
@@ -193,31 +190,20 @@ class Manager(manager.Manager):
         self.revoke(revoke_model.RevokeEvent(domain_id=domain_id,
                                              role_id=role_id))
 
-    def check_token(self, token_values):
+    def check_token(self, token):
         """Check the values from a token against the revocation list.
 
-        :param token_values: dictionary of values from a token, normalized for
+        :param token: dictionary of values from a token, normalized for
                              differences between v2 and v3. The checked values
                              are a subset of the attributes of model.TokenEvent
 
         :raises keystone.exception.TokenNotFound: If the token is invalid.
 
         """
-        if revoke_model.is_revoked(self.list_events(), token_values):
+        if revoke_model.is_revoked(self.driver.list_events(token=token),
+                                   token):
             raise exception.TokenNotFound(_('Failed to validate token'))
 
     def revoke(self, event):
         self.driver.revoke(event)
         REVOKE_REGION.invalidate()
-
-
-@versionutils.deprecated(
-    versionutils.deprecated.NEWTON,
-    what='keystone.revoke.RevokeDriverV8',
-    in_favor_of='keystone.revoke.backends.base.RevokeDriverV8',
-    remove_in=+1)
-class RevokeDriverV8(base.RevokeDriverV8):
-    pass
-
-
-Driver = manager.create_legacy_driver(base.RevokeDriverV8)
