@@ -19,7 +19,7 @@ import datetime
 import functools
 import hashlib
 import json
-import logging
+import ldap
 import os
 import re
 import shutil
@@ -29,40 +29,33 @@ import uuid
 import warnings
 
 import fixtures
-from oslo_config import cfg
 from oslo_config import fixture as config_fixture
 from oslo_context import context as oslo_context
 from oslo_context import fixture as oslo_ctx_fixture
 from oslo_log import fixture as log_fixture
 from oslo_log import log
 from oslo_utils import timeutils
-from oslotest import mockpatch
 from paste.deploy import loadwsgi
 import six
 from sqlalchemy import exc
 import testtools
 from testtools import testcase
 
-# NOTE(ayoung)
-# environment.use_eventlet must run before any of the code that will
-# call the eventlet monkeypatching.
-from keystone.common import environment  # noqa
-environment.use_eventlet()
-
-from keystone import auth
-from keystone.common import config
+from keystone.common import context
 from keystone.common import dependency
-from keystone.common.kvs import core as kvs_core
+from keystone.common import request
 from keystone.common import sql
+import keystone.conf
 from keystone import exception
+from keystone.identity.backends.ldap import common as ks_ldap
 from keystone import notifications
-from keystone.server import common
 from keystone.tests.unit import ksfixtures
 from keystone.version import controllers
 from keystone.version import service
 
 
-config.configure()
+keystone.conf.configure()
+keystone.conf.set_config_defaults()
 
 PID = six.text_type(os.getpid())
 TESTSDIR = os.path.dirname(os.path.abspath(__file__))
@@ -81,12 +74,15 @@ def _calc_tmpdir():
 
 TMPDIR = _calc_tmpdir()
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 log.register_options(CONF)
 
 IN_MEM_DB_CONN_STRING = 'sqlite://'
 
+# Strictly matches ISO 8601 timestamps with subsecond precision like:
+# 2016-06-28T20:48:56.000000Z
 TIME_FORMAT = '%Y-%m-%dT%H:%M:%S.%fZ'
+TIME_FORMAT_REGEX = '^\d{4}-[0-1]\d-[0-3]\dT[0-2]\d:[0-5]\d:[0-5]\d\.\d{6}Z$'
 
 exception._FATAL_EXCEPTION_FORMAT_ERRORS = True
 os.makedirs(TMPDIR)
@@ -115,10 +111,6 @@ class dirs(object):
         return os.path.join(TESTCONF, *p)
 
 
-# keystone.common.sql.initialize() for testing.
-DEFAULT_TEST_DB_FILE = dirs.tmp('test.db')
-
-
 class EggLoader(loadwsgi.EggLoader):
     _basket = {}
 
@@ -149,30 +141,8 @@ def remove_test_databases():
         os.unlink(pristine)
 
 
-def generate_paste_config(extension_name):
-    # Generate a file, based on keystone-paste.ini, that is named:
-    # extension_name.ini, and includes extension_name in the pipeline
-    with open(dirs.etc('keystone-paste.ini'), 'r') as f:
-        contents = f.read()
-
-    new_contents = contents.replace(' service_v3',
-                                    ' %s service_v3' % (extension_name))
-
-    new_paste_file = dirs.tmp(extension_name + '.ini')
-    with open(new_paste_file, 'w') as f:
-        f.write(new_contents)
-
-    return new_paste_file
-
-
-def remove_generated_paste_config(extension_name):
-    # Remove the generated paste config file, named extension_name.ini
-    paste_file_to_remove = dirs.tmp(extension_name + '.ini')
-    os.remove(paste_file_to_remove)
-
-
 def skip_if_cache_disabled(*sections):
-    """This decorator is used to skip a test if caching is disabled.
+    """Skip a test if caching is disabled, this is a decorator.
 
     Caching can be disabled either globally or for a specific section.
 
@@ -414,7 +384,9 @@ def new_ec2_credential(user_id, project_id=None, blob=None, **kwargs):
 
 def new_totp_credential(user_id, project_id=None, blob=None):
     if not blob:
-        blob = base64.b32encode(uuid.uuid4().hex).rstrip('=')
+        # NOTE(notmorgan): 20 bytes of data from os.urandom for
+        # a totp secret.
+        blob = base64.b32encode(os.urandom(20)).decode('utf-8')
     credential = new_credential_ref(user_id=user_id,
                                     project_id=project_id,
                                     blob=blob,
@@ -517,8 +489,8 @@ class BaseTestCase(testtools.TestCase):
         self.useFixture(fixtures.NestedTempfile())
         self.useFixture(fixtures.TempHomeDir())
 
-        self.useFixture(mockpatch.PatchObject(sys, 'exit',
-                                              side_effect=UnexpectedExit))
+        self.useFixture(fixtures.MockPatchObject(sys, 'exit',
+                                                 side_effect=UnexpectedExit))
         self.useFixture(log_fixture.get_logging_handle_error_fixture())
 
         warnings.filterwarnings('error', category=DeprecationWarning,
@@ -530,6 +502,25 @@ class BaseTestCase(testtools.TestCase):
         self.assertIsNone(oslo_context.get_current())
         self.useFixture(oslo_ctx_fixture.ClearRequestContext())
 
+        orig_debug_level = ldap.get_option(ldap.OPT_DEBUG_LEVEL)
+        self.addCleanup(ldap.set_option, ldap.OPT_DEBUG_LEVEL,
+                        orig_debug_level)
+        orig_tls_cacertfile = ldap.get_option(ldap.OPT_X_TLS_CACERTFILE)
+        if orig_tls_cacertfile is None:
+            orig_tls_cacertfile = ''
+        self.addCleanup(ldap.set_option, ldap.OPT_X_TLS_CACERTFILE,
+                        orig_tls_cacertfile)
+        orig_tls_cacertdir = ldap.get_option(ldap.OPT_X_TLS_CACERTDIR)
+        # Setting orig_tls_cacertdir to None is not allowed.
+        if orig_tls_cacertdir is None:
+            orig_tls_cacertdir = ''
+        self.addCleanup(ldap.set_option, ldap.OPT_X_TLS_CACERTDIR,
+                        orig_tls_cacertdir)
+        orig_tls_require_cert = ldap.get_option(ldap.OPT_X_TLS_REQUIRE_CERT)
+        self.addCleanup(ldap.set_option, ldap.OPT_X_TLS_REQUIRE_CERT,
+                        orig_tls_require_cert)
+        self.addCleanup(ks_ldap.PooledLDAPHandler.connection_pools.clear)
+
     def cleanup_instance(self, *names):
         """Create a function suitable for use with self.addCleanup.
 
@@ -545,6 +536,23 @@ class BaseTestCase(testtools.TestCase):
                     delattr(self, name)
         return cleanup
 
+    def skip_if_env_not_set(self, env_var):
+        if not os.environ.get(env_var):
+            self.skipTest('Env variable %s is not set.' % env_var)
+
+    def skip_test_overrides(self, *args, **kwargs):
+        if self._check_for_method_in_parents(self._testMethodName):
+            return super(BaseTestCase, self).skipTest(*args, **kwargs)
+        raise Exception('%r is not a previously defined test method'
+                        % self._testMethodName)
+
+    def _check_for_method_in_parents(self, name):
+        # skip first to get to parents
+        for cls in self.__class__.__mro__[1:]:
+            if hasattr(cls, name):
+                return True
+        return False
+
 
 class TestCase(BaseTestCase):
 
@@ -554,6 +562,20 @@ class TestCase(BaseTestCase):
     def _policy_fixture(self):
         return ksfixtures.Policy(dirs.etc('policy.json'), self.config_fixture)
 
+    def make_request(self, path='/', **kwargs):
+        is_admin = kwargs.pop('is_admin', False)
+        environ = kwargs.setdefault('environ', {})
+
+        if not environ.get(context.REQUEST_CONTEXT_ENV):
+            environ[context.REQUEST_CONTEXT_ENV] = context.RequestContext(
+                is_admin=is_admin,
+                authenticated=kwargs.pop('authenticated', True))
+
+        req = request.Request.blank(path=path, **kwargs)
+        req.context_dict['is_admin'] = is_admin
+
+        return req
+
     def config_overrides(self):
         # NOTE(morganfainberg): enforce config_overrides can only ever be
         # called a single time.
@@ -586,7 +608,6 @@ class TestCase(BaseTestCase):
             group='signing', certfile=signing_certfile,
             keyfile=signing_keyfile,
             ca_certs='examples/pki/certs/cacert.pem')
-        self.config_fixture.config(group='token', driver='kvs')
         self.config_fixture.config(
             group='saml', certfile=signing_certfile, keyfile=signing_keyfile)
         self.config_fixture.config(
@@ -603,15 +624,15 @@ class TestCase(BaseTestCase):
                 'routes.middleware=INFO',
                 'stevedore.extension=INFO',
                 'keystone.notifications=INFO',
-                'keystone.common.ldap=INFO',
+                'keystone.identity.backends.ldap.common=INFO',
             ])
-        self.auth_plugin_config_override()
-
-    def auth_plugin_config_override(self, methods=None, **method_classes):
         self.useFixture(
-            ksfixtures.ConfigAuthPlugins(self.config_fixture,
-                                         methods,
-                                         **method_classes))
+            ksfixtures.KeyRepository(
+                self.config_fixture,
+                'fernet_tokens',
+                CONF.fernet_tokens.max_active_keys
+            )
+        )
 
     def _assert_config_overrides_called(self):
         assert self.__config_overrides_called is True
@@ -620,7 +641,6 @@ class TestCase(BaseTestCase):
         super(TestCase, self).setUp()
         self.__config_overrides_called = False
         self.__load_backends_called = False
-        self.addCleanup(CONF.reset)
         self.config_fixture = self.useFixture(config_fixture.Config(CONF))
         self.addCleanup(delattr, self, 'config_fixture')
         self.config(self.config_files())
@@ -630,26 +650,25 @@ class TestCase(BaseTestCase):
         # cleanup.
         def mocked_register_auth_plugin_opt(conf, opt):
             self.config_fixture.register_opt(opt, group='auth')
-        self.useFixture(mockpatch.PatchObject(
-            config, '_register_auth_plugin_opt',
+        self.useFixture(fixtures.MockPatchObject(
+            keystone.conf.auth, '_register_auth_plugin_opt',
             new=mocked_register_auth_plugin_opt))
 
-        self.sql_driver_version_overrides = {}
         self.config_overrides()
         # NOTE(morganfainberg): ensure config_overrides has been called.
         self.addCleanup(self._assert_config_overrides_called)
 
-        self.useFixture(fixtures.FakeLogger(level=logging.DEBUG))
+        self.useFixture(fixtures.FakeLogger(level=log.DEBUG))
 
         # NOTE(morganfainberg): This code is a copy from the oslo-incubator
         # log module. This is not in a function or otherwise available to use
         # without having a CONF object to setup logging. This should help to
         # reduce the log size by limiting what we log (similar to how Keystone
-        # would run under mod_wsgi or eventlet).
+        # would run under mod_wsgi).
         for pair in CONF.default_log_levels:
             mod, _sep, level_name = pair.partition('=')
-            logger = logging.getLogger(mod)
-            logger.setLevel(level_name)
+            logger = log.getLogger(mod)
+            logger.logger.setLevel(level_name)
 
         self.useFixture(ksfixtures.Cache())
 
@@ -661,9 +680,6 @@ class TestCase(BaseTestCase):
         self.addCleanup(notifications.clear_subscribers)
         self.addCleanup(notifications.reset_notifier)
 
-        # Reset the auth-plugin registry
-        self.addCleanup(self.clear_auth_plugin_registry)
-
         self.addCleanup(setattr, controllers, '_VERSIONS', [])
 
     def config(self, config_files):
@@ -671,35 +687,8 @@ class TestCase(BaseTestCase):
         CONF(args=[], project='keystone', default_config_files=config_files)
 
     def load_backends(self):
-        """Initializes each manager and assigns them to an attribute."""
-        # TODO(blk-u): Shouldn't need to clear the registry here, but some
-        # tests call load_backends multiple times. These should be fixed to
-        # only call load_backends once.
-        dependency.reset()
-
-        # TODO(morganfainberg): Shouldn't need to clear the registry here, but
-        # some tests call load_backends multiple times.  Since it is not
-        # possible to re-configure a backend, we need to clear the list.  This
-        # should eventually be removed once testing has been cleaned up.
-        kvs_core.KEY_VALUE_STORE_REGISTRY.clear()
-
-        self.clear_auth_plugin_registry()
-        drivers, _unused = common.setup_backends(
-            load_extra_backends_fn=self.load_extra_backends)
-
-        for manager_name, manager in drivers.items():
-            setattr(self, manager_name, manager)
-        self.addCleanup(self.cleanup_instance(*list(drivers.keys())))
-
-    def load_extra_backends(self):
-        """Override to load managers that aren't loaded by default.
-
-        This is useful to load managers initialized by extensions. No extra
-        backends are loaded by default.
-
-        :returns: dict of name -> manager
-        """
-        return {}
+        """Initialize each manager and assigns them to an attribute."""
+        self.useFixture(ksfixtures.BackendLoader(self))
 
     def load_fixtures(self, fixtures):
         """Hacky basic and naive fixture loading based on a python module.
@@ -806,12 +795,8 @@ class TestCase(BaseTestCase):
     def loadapp(self, config, name='main'):
         return service.loadapp(self._paste_config(config), name=name)
 
-    def clear_auth_plugin_registry(self):
-        auth.controllers.AUTH_METHODS.clear()
-        auth.controllers.AUTH_PLUGINS_LOADED = False
-
     def assertCloseEnoughForGovernmentWork(self, a, b, delta=3):
-        """Asserts that two datetimes are nearly equal within a small delta.
+        """Assert that two datetimes are nearly equal within a small delta.
 
         :param delta: Maximum allowable time delta, defined in seconds.
         """
@@ -821,14 +806,25 @@ class TestCase(BaseTestCase):
 
         msg = '%s != %s within %s delta' % (a, b, delta)
 
-        self.assertTrue(abs(a - b).seconds <= delta, msg)
+        self.assertLessEqual(abs(a - b).seconds, delta, msg)
+
+    def assertTimestampEqual(self, expected, value):
+        # Compare two timestamps but ignore the microseconds part
+        # of the expected timestamp. Keystone does not track microseconds and
+        # is working to eliminate microseconds from it's datetimes used.
+        expected = timeutils.parse_isotime(expected).replace(microsecond=0)
+        value = timeutils.parse_isotime(value).replace(microsecond=0)
+        self.assertEqual(
+            expected,
+            value,
+            "%s != %s" % (expected, value))
 
     def assertNotEmpty(self, l):
-        self.assertTrue(len(l))
+        self.assertGreater(len(l), 0)
 
     def assertRaisesRegexp(self, expected_exception, expected_regexp,
                            callable_obj, *args, **kwargs):
-        """Asserts that the message in a raised exception matches a regexp."""
+        """Assert that the message in a raised exception matches a regexp."""
         try:
             callable_obj(*args, **kwargs)
         except expected_exception as exc_value:
@@ -852,6 +848,24 @@ class TestCase(BaseTestCase):
                 excName = str(expected_exception)
             raise self.failureException("%s not raised" % excName)
 
+    def assertUserDictEqual(self, expected, observed, message=''):
+        """Assert that a user dict is equal to another user dict.
+
+        User dictionaries have some variable values that should be ignored in
+        the comparison. This method is a helper that strips those elements out
+        when comparing the user dictionary. This normalized these differences
+        that should not change the comparison.
+        """
+        # NOTE(notmorgan): An empty option list is the same as no options being
+        # specified in the user_ref. This removes options if it is empty in
+        # observed if options is not specified in the expected value.
+        if ('options' in observed and not observed['options'] and
+                'options' not in expected):
+            observed = observed.copy()
+            del observed['options']
+
+        self.assertDictEqual(expected, observed, message)
+
     @property
     def ipv6_enabled(self):
         if socket.has_ipv6:
@@ -872,10 +886,6 @@ class TestCase(BaseTestCase):
         if not self.ipv6_enabled:
             raise self.skipTest("IPv6 is not enabled in the system")
 
-    def skip_if_env_not_set(self, env_var):
-        if not os.environ.get(env_var):
-            self.skipTest('Env variable %s is not set.' % env_var)
-
 
 class SQLDriverOverrides(object):
     """A mixin for consolidating sql-specific test overrides."""
@@ -888,20 +898,3 @@ class SQLDriverOverrides(object):
         self.config_fixture.config(group='policy', driver='sql')
         self.config_fixture.config(group='token', driver='sql')
         self.config_fixture.config(group='trust', driver='sql')
-
-    def use_specific_sql_driver_version(self, driver_path,
-                                        versionless_backend, version_suffix):
-        """Add this versioned driver to the list that will be loaded.
-
-        :param driver_path: The path to the drivers, e.g. 'keystone.assignment'
-        :param versionless_backend: The name of the versionless drivers, e.g.
-                                    'backends'
-        :param version_suffix: The suffix for the version , e.g. ``V8_``
-
-        This method assumes that versioned drivers are named:
-        <version_suffix><name of versionless driver>, e.g. 'V8_backends'.
-
-        """
-        self.sql_driver_version_overrides[driver_path] = {
-            'versionless_backend': versionless_backend,
-            'versioned_backend': version_suffix + versionless_backend}
