@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,9 +12,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import datetime
 import functools
 import uuid
 
+import freezegun
 import mock
 from oslo_db import exception as db_exception
 from oslo_db import options
@@ -52,7 +53,7 @@ class SqlTests(unit.SQLDriverOverrides, unit.TestCase):
 
     def setUp(self):
         super(SqlTests, self).setUp()
-        self.useFixture(database.Database(self.sql_driver_version_overrides))
+        self.useFixture(database.Database())
         self.load_backends()
 
         # populate the engine with tables & fixtures
@@ -132,6 +133,7 @@ class SqlModels(SqlTests):
 
     def test_user_model(self):
         cols = (('id', sql.String, 64),
+                ('domain_id', sql.String, 64),
                 ('default_project_id', sql.String, 64),
                 ('enabled', sql.Boolean, None),
                 ('extra', sql.JsonBlob, None),
@@ -223,7 +225,6 @@ class SqlModels(SqlTests):
 
 class SqlIdentity(SqlTests,
                   identity_tests.IdentityTests,
-                  identity_tests.ShadowUsersTests,
                   assignment_tests.AssignmentTests,
                   resource_tests.ResourceTests):
     def test_password_hashed(self):
@@ -735,6 +736,30 @@ class SqlToken(SqlTests, token_tests.TokenTests):
         self.assertEqual(token_sql._expiry_range_batched, mysql_strategy.func)
         self.assertEqual({'batch_size': 1000}, mysql_strategy.keywords)
 
+    def test_expiry_range_with_allow_expired(self):
+        window_secs = 200
+        self.config_fixture.config(group='token',
+                                   allow_expired_window=window_secs)
+
+        tok = token_sql.Token()
+        time = datetime.datetime.utcnow()
+
+        with freezegun.freeze_time(time):
+            # unknown strategy just ensures we are getting the dumbest strategy
+            # that will remove everything in one go
+            strategy = tok._expiry_range_strategy('unkown')
+            upper_bound_func = token_sql._expiry_upper_bound_func
+
+            # session is ignored for dumb strategy
+            expiry_times = list(strategy(session=None,
+                                         upper_bound_func=upper_bound_func))
+
+            # basically just ensure that we are removing things in the past
+            delta = datetime.timedelta(seconds=window_secs)
+            previous_time = datetime.datetime.utcnow() - delta
+
+        self.assertEqual([previous_time], expiry_times)
+
 
 class SqlCatalog(SqlTests, catalog_tests.CatalogTests):
 
@@ -827,6 +852,70 @@ class SqlCatalog(SqlTests, catalog_tests.CatalogTests):
                           self.catalog_api.delete_region,
                           region['id'])
 
+    def test_v3_catalog_domain_scoped_token(self):
+        # test the case that tenant_id is None.
+        srv_1 = unit.new_service_ref()
+        self.catalog_api.create_service(srv_1['id'], srv_1)
+        endpoint_1 = unit.new_endpoint_ref(service_id=srv_1['id'],
+                                           region_id=None)
+        self.catalog_api.create_endpoint(endpoint_1['id'], endpoint_1)
+
+        srv_2 = unit.new_service_ref()
+        self.catalog_api.create_service(srv_2['id'], srv_2)
+        endpoint_2 = unit.new_endpoint_ref(service_id=srv_2['id'],
+                                           region_id=None)
+        self.catalog_api.create_endpoint(endpoint_2['id'], endpoint_2)
+
+        self.config_fixture.config(group='endpoint_filter',
+                                   return_all_endpoints_if_no_filter=True)
+        catalog_ref = self.catalog_api.get_v3_catalog(uuid.uuid4().hex, None)
+        self.assertThat(catalog_ref, matchers.HasLength(2))
+        self.config_fixture.config(group='endpoint_filter',
+                                   return_all_endpoints_if_no_filter=False)
+        catalog_ref = self.catalog_api.get_v3_catalog(uuid.uuid4().hex, None)
+        self.assertThat(catalog_ref, matchers.HasLength(0))
+
+    def test_v3_catalog_endpoint_filter_enabled(self):
+        srv_1 = unit.new_service_ref()
+        self.catalog_api.create_service(srv_1['id'], srv_1)
+        endpoint_1 = unit.new_endpoint_ref(service_id=srv_1['id'],
+                                           region_id=None)
+        self.catalog_api.create_endpoint(endpoint_1['id'], endpoint_1)
+        endpoint_2 = unit.new_endpoint_ref(service_id=srv_1['id'],
+                                           region_id=None)
+        self.catalog_api.create_endpoint(endpoint_2['id'], endpoint_2)
+        # create endpoint-project association.
+        self.catalog_api.add_endpoint_to_project(
+            endpoint_1['id'],
+            self.tenant_bar['id'])
+
+        catalog_ref = self.catalog_api.get_v3_catalog(uuid.uuid4().hex,
+                                                      self.tenant_bar['id'])
+        self.assertThat(catalog_ref, matchers.HasLength(1))
+        self.assertThat(catalog_ref[0]['endpoints'], matchers.HasLength(1))
+        # the endpoint is that defined in the endpoint-project association.
+        self.assertEqual(endpoint_1['id'],
+                         catalog_ref[0]['endpoints'][0]['id'])
+
+    def test_v3_catalog_endpoint_filter_disabled(self):
+        # there is no endpoint-project association defined.
+        self.config_fixture.config(group='endpoint_filter',
+                                   return_all_endpoints_if_no_filter=True)
+        srv_1 = unit.new_service_ref()
+        self.catalog_api.create_service(srv_1['id'], srv_1)
+        endpoint_1 = unit.new_endpoint_ref(service_id=srv_1['id'],
+                                           region_id=None)
+        self.catalog_api.create_endpoint(endpoint_1['id'], endpoint_1)
+
+        srv_2 = unit.new_service_ref()
+        self.catalog_api.create_service(srv_2['id'], srv_2)
+
+        catalog_ref = self.catalog_api.get_v3_catalog(uuid.uuid4().hex,
+                                                      self.tenant_bar['id'])
+        self.assertThat(catalog_ref, matchers.HasLength(2))
+        srv_id_list = [catalog_ref[0]['id'], catalog_ref[1]['id']]
+        self.assertItemsEqual([srv_1['id'], srv_2['id']], srv_id_list)
+
 
 class SqlPolicy(SqlTests, policy_tests.PolicyTests):
     pass
@@ -854,33 +943,6 @@ class SqlTokenCacheInvalidationWithUUID(SqlTests,
         self.config_fixture.config(group='token', provider='uuid')
 
 
-class SqlTokenCacheInvalidationWithPKI(SqlTests,
-                                       token_tests.TokenCacheInvalidation):
-    def setUp(self):
-        super(SqlTokenCacheInvalidationWithPKI, self).setUp()
-        self._create_test_data()
-
-    def config_overrides(self):
-        super(SqlTokenCacheInvalidationWithPKI, self).config_overrides()
-        # NOTE(lbragstad): The TokenCacheInvalidation tests are coded to work
-        # against a persistent token backend. Only run these with token
-        # providers that issue persistent tokens.
-        self.config_fixture.config(group='token', provider='pki')
-
-
-class SqlTokenCacheInvalidationWithPKIZ(SqlTests,
-                                        token_tests.TokenCacheInvalidation):
-    def setUp(self):
-        super(SqlTokenCacheInvalidationWithPKIZ, self).setUp()
-        self._create_test_data()
-
-    def config_overrides(self):
-        super(SqlTokenCacheInvalidationWithPKIZ, self).config_overrides()
-        # NOTE(lbragstad): The TokenCacheInvalidation tests are coded to work
-        # against a persistent token backend. Only run these with token
-        # providers that issue persistent tokens.
-        self.config_fixture.config(group='token', provider='pkiz')
-
 # NOTE(lbragstad): The Fernet token provider doesn't persist tokens in a
 # backend, so running the TokenCacheInvalidation tests here doesn't make sense.
 
