@@ -14,23 +14,21 @@
 
 import datetime
 
-from oslo_config import cfg
 from oslo_utils import timeutils
-from six.moves import reload_module
 
 from keystone.common import dependency
 from keystone.common import utils
+import keystone.conf
 from keystone import exception
 from keystone.tests import unit
+from keystone.tests.unit import ksfixtures
 from keystone.tests.unit.ksfixtures import database
 from keystone import token
 from keystone.token.providers import fernet
-from keystone.token.providers import pki
-from keystone.token.providers import pkiz
 from keystone.token.providers import uuid
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 
 FUTURE_DELTA = datetime.timedelta(seconds=CONF.token.expiration)
 CURRENT_DATE = timeutils.utcnow()
@@ -317,6 +315,7 @@ SAMPLE_V3_TOKEN = {
             "id": "01257",
             "name": "service"
         },
+        "is_domain": False,
         "roles": [
             {
                 "id": "9fe2ff9ee4384b1894a90878d3e92bab",
@@ -333,7 +332,8 @@ SAMPLE_V3_TOKEN = {
                 "name": "Default"
             },
             "id": "f19ddbe2c53c46f189fe66d0a7a9c9ce",
-            "name": "nova"
+            "name": "nova",
+            "password_expires_at": "2013-08-21T00:02:43.941473Z"
         },
         "OS-TRUST:trust": {
             "id": "abc123",
@@ -717,6 +717,13 @@ class TestTokenProvider(unit.TestCase):
     def setUp(self):
         super(TestTokenProvider, self).setUp()
         self.useFixture(database.Database())
+        self.useFixture(
+            ksfixtures.KeyRepository(
+                self.config_fixture,
+                'fernet_tokens',
+                CONF.fernet_tokens.max_active_keys
+            )
+        )
         self.load_backends()
 
     def test_get_token_version(self):
@@ -743,20 +750,12 @@ class TestTokenProvider(unit.TestCase):
 
         dependency.reset()
         self.assertIsInstance(token.provider.Manager().driver,
-                              uuid.Provider)
+                              fernet.Provider)
 
         dependency.reset()
         self.config_fixture.config(group='token', provider='uuid')
         self.assertIsInstance(token.provider.Manager().driver, uuid.Provider)
 
-        dependency.reset()
-        self.config_fixture.config(group='token', provider='pki')
-        self.assertIsInstance(token.provider.Manager().driver, pki.Provider)
-
-        dependency.reset()
-        self.config_fixture.config(group='token', provider='pkiz')
-        self.assertIsInstance(token.provider.Manager().driver, pkiz.Provider)
-
         dependency.reset()
         self.config_fixture.config(group='token', provider='fernet')
         self.assertIsInstance(token.provider.Manager().driver, fernet.Provider)
@@ -782,64 +781,8 @@ class TestTokenProvider(unit.TestCase):
         self.assertIsNone(
             self.token_provider_api._is_valid_token(create_v3_token()))
 
-    def test_no_token_raises_token_not_found(self):
+    def test_validate_v3_token_with_no_token_raises_token_not_found(self):
         self.assertRaises(
             exception.TokenNotFound,
             self.token_provider_api.validate_token,
             None)
-
-
-# NOTE(ayoung): renamed to avoid automatic test detection
-class PKIProviderTests(object):
-
-    def setUp(self):
-        super(PKIProviderTests, self).setUp()
-
-        from keystoneclient.common import cms
-        self.cms = cms
-
-        from keystone.common import environment
-        self.environment = environment
-
-        old_cms_subprocess = cms.subprocess
-        self.addCleanup(setattr, cms, 'subprocess', old_cms_subprocess)
-
-        old_env_subprocess = environment.subprocess
-        self.addCleanup(setattr, environment, 'subprocess', old_env_subprocess)
-
-        self.cms.subprocess = self.target_subprocess
-        self.environment.subprocess = self.target_subprocess
-
-        # force module reload so the imports get re-evaluated
-        reload_module(pki)
-
-    def test_get_token_id_error_handling(self):
-        # cause command-line failure
-        self.config_fixture.config(group='signing',
-                                   keyfile='--please-break-me')
-
-        provider = pki.Provider()
-        token_data = {}
-        self.assertRaises(exception.UnexpectedError,
-                          provider._get_token_id,
-                          token_data)
-
-
-class TestPKIProviderWithEventlet(PKIProviderTests, unit.TestCase):
-
-    def setUp(self):
-        # force keystoneclient.common.cms to use eventlet's subprocess
-        from eventlet.green import subprocess
-        self.target_subprocess = subprocess
-
-        super(TestPKIProviderWithEventlet, self).setUp()
-
-
-class TestPKIProviderWithStdlib(PKIProviderTests, unit.TestCase):
-
-    def setUp(self):
-        # force keystoneclient.common.cms to use the stdlib subprocess
-        import subprocess
-        self.target_subprocess = subprocess
-
-        super(TestPKIProviderWithStdlib, self).setUp()
