@@ -13,33 +13,42 @@
 import base64
 import datetime
 import hashlib
+import mock
 import os
 import uuid
 
 import msgpack
 from oslo_utils import timeutils
+import six
 from six.moves import urllib
 
-from keystone.common import config
+from keystone import auth
+from keystone.common import fernet_utils
 from keystone.common import utils
+import keystone.conf
 from keystone import exception
 from keystone.federation import constants as federation_constants
 from keystone.tests import unit
 from keystone.tests.unit import ksfixtures
 from keystone.tests.unit.ksfixtures import database
-from keystone.token import provider
+from keystone.token.providers import common
 from keystone.token.providers import fernet
 from keystone.token.providers.fernet import token_formatters
-from keystone.token.providers.fernet import utils as fernet_utils
 
 
-CONF = config.CONF
+CONF = keystone.conf.CONF
 
 
 class TestFernetTokenProvider(unit.TestCase):
     def setUp(self):
         super(TestFernetTokenProvider, self).setUp()
-        self.useFixture(ksfixtures.KeyRepository(self.config_fixture))
+        self.useFixture(
+            ksfixtures.KeyRepository(
+                self.config_fixture,
+                'fernet_tokens',
+                CONF.fernet_tokens.max_active_keys
+            )
+        )
         self.provider = fernet.Provider()
 
     def test_supports_bind_authentication_returns_false(self):
@@ -49,14 +58,10 @@ class TestFernetTokenProvider(unit.TestCase):
         self.assertFalse(self.provider.needs_persistence())
 
     def test_invalid_v3_token_raises_token_not_found(self):
-        # NOTE(lbragstad): Here we use the validate_non_persistent_token()
-        # methods because the validate_v3_token() method is strictly for
-        # validating UUID formatted tokens. It is written to assume cached
-        # tokens from a backend, where validate_non_persistent_token() is not.
         token_id = uuid.uuid4().hex
         e = self.assertRaises(
             exception.TokenNotFound,
-            self.provider.validate_non_persistent_token,
+            self.provider.validate_token,
             token_id)
         self.assertIn(token_id, u'%s' % e)
 
@@ -64,7 +69,7 @@ class TestFernetTokenProvider(unit.TestCase):
         token_id = uuid.uuid4().hex
         e = self.assertRaises(
             exception.TokenNotFound,
-            self.provider.validate_non_persistent_token,
+            self.provider.validate_token,
             token_id)
         self.assertIn(token_id, u'%s' % e)
 
@@ -72,7 +77,13 @@ class TestFernetTokenProvider(unit.TestCase):
 class TestValidate(unit.TestCase):
     def setUp(self):
         super(TestValidate, self).setUp()
-        self.useFixture(ksfixtures.KeyRepository(self.config_fixture))
+        self.useFixture(
+            ksfixtures.KeyRepository(
+                self.config_fixture,
+                'fernet_tokens',
+                CONF.fernet_tokens.max_active_keys
+            )
+        )
         self.useFixture(database.Database())
         self.load_backends()
 
@@ -92,10 +103,10 @@ class TestValidate(unit.TestCase):
         user_ref = self.identity_api.create_user(user_ref)
 
         method_names = ['password']
-        token_id, token_data_ = self.token_provider_api.issue_v3_token(
+        token_id, token_data_ = self.token_provider_api.issue_token(
             user_ref['id'], method_names)
 
-        token_data = self.token_provider_api.validate_v3_token(token_id)
+        token_data = self.token_provider_api.validate_token(token_id)
         token = token_data['token']
         self.assertIsInstance(token['audit_ids'], list)
         self.assertIsInstance(token['expires_at'], str)
@@ -108,6 +119,7 @@ class TestValidate(unit.TestCase):
                 'id': domain_ref['id'],
                 'name': domain_ref['name'],
             },
+            'password_expires_at': user_ref['password_expires_at']
         }
         self.assertEqual(exp_user_info, token['user'])
 
@@ -127,17 +139,18 @@ class TestValidate(unit.TestCase):
         group_ids = [uuid.uuid4().hex, ]
         identity_provider = uuid.uuid4().hex
         protocol = uuid.uuid4().hex
-        auth_context = {
+        auth_context_params = {
             'user_id': user_ref['id'],
             'user_name': user_ref['name'],
             'group_ids': group_ids,
             federation_constants.IDENTITY_PROVIDER: identity_provider,
             federation_constants.PROTOCOL: protocol,
         }
-        token_id, token_data_ = self.token_provider_api.issue_v3_token(
+        auth_context = auth.core.AuthContext(**auth_context_params)
+        token_id, token_data_ = self.token_provider_api.issue_token(
             user_ref['id'], method_names, auth_context=auth_context)
 
-        token_data = self.token_provider_api.validate_v3_token(token_id)
+        token_data = self.token_provider_api.validate_token(token_id)
         token = token_data['token']
         exp_user_info = {
             'id': user_ref['id'],
@@ -150,7 +163,7 @@ class TestValidate(unit.TestCase):
                 'protocol': {'id': protocol, },
             },
         }
-        self.assertEqual(exp_user_info, token['user'])
+        self.assertDictEqual(exp_user_info, token['user'])
 
     def test_validate_v3_token_trust(self):
         # Check the trust fields in the token result when use validate_v3_token
@@ -191,11 +204,11 @@ class TestValidate(unit.TestCase):
 
         method_names = ['password']
 
-        token_id, token_data_ = self.token_provider_api.issue_v3_token(
+        token_id, token_data_ = self.token_provider_api.issue_token(
             user_ref['id'], method_names, project_id=project_ref['id'],
             trust=trust_ref)
 
-        token_data = self.token_provider_api.validate_v3_token(token_id)
+        token_data = self.token_provider_api.validate_token(token_id)
         token = token_data['token']
         exp_trust_info = {
             'id': trust_ref['id'],
@@ -211,13 +224,19 @@ class TestValidate(unit.TestCase):
         # A uuid string isn't a valid Fernet token.
         token_id = uuid.uuid4().hex
         self.assertRaises(exception.TokenNotFound,
-                          self.token_provider_api.validate_v3_token, token_id)
+                          self.token_provider_api.validate_token, token_id)
 
 
 class TestTokenFormatter(unit.TestCase):
     def setUp(self):
         super(TestTokenFormatter, self).setUp()
-        self.useFixture(ksfixtures.KeyRepository(self.config_fixture))
+        self.useFixture(
+            ksfixtures.KeyRepository(
+                self.config_fixture,
+                'fernet_tokens',
+                CONF.fernet_tokens.max_active_keys
+            )
+        )
 
     def test_restore_padding(self):
         # 'a' will result in '==' padding, 'aa' will result in '=' padding, and
@@ -294,6 +313,13 @@ class TestPayloads(unit.TestCase):
         return self.assertCloseEnoughForGovernmentWork(exp_time, actual_time,
                                                        delta=1e-05)
 
+    def test_strings_can_be_converted_to_bytes(self):
+        s = common.random_urlsafe_str()
+        self.assertIsInstance(s, six.text_type)
+
+        b = token_formatters.BasePayload.random_urlsafe_str_to_bytes(s)
+        self.assertIsInstance(b, six.binary_type)
+
     def test_uuid_hex_to_byte_conversions(self):
         payload_cls = token_formatters.BasePayload
 
@@ -344,7 +370,7 @@ class TestPayloads(unit.TestCase):
         exp_user_id = exp_user_id or uuid.uuid4().hex
         exp_methods = exp_methods or ['password']
         exp_expires_at = utils.isotime(timeutils.utcnow(), subsecond=True)
-        exp_audit_ids = [provider.random_urlsafe_str()]
+        exp_audit_ids = [common.random_urlsafe_str()]
 
         payload = payload_class.assemble(
             exp_user_id, exp_methods, exp_project_id, exp_domain_id,
@@ -507,7 +533,12 @@ class TestFernetKeyRotation(unit.TestCase):
 
         """
         # Load the keys into a list, keys is list of six.text_type.
-        keys = fernet_utils.load_keys()
+        key_utils = fernet_utils.FernetUtils(
+            CONF.fernet_tokens.key_repository,
+            CONF.fernet_tokens.max_active_keys,
+            'fernet_tokens'
+        )
+        keys = key_utils.load_keys()
 
         # Sort the list of keys by the keys themselves (they were previously
         # sorted by filename).
@@ -526,7 +557,7 @@ class TestFernetKeyRotation(unit.TestCase):
         self.assertUniqueRepositoryState()
 
     def assertUniqueRepositoryState(self):
-        """Ensures that the current key repo state has not been seen before."""
+        """Ensure that the current key repo state has not been seen before."""
         # This is assigned to a variable because it takes some work to
         # calculate.
         signature = self.key_repository_signature
@@ -551,7 +582,13 @@ class TestFernetKeyRotation(unit.TestCase):
 
             # Ensure that resetting the key repository always results in 2
             # active keys.
-            self.useFixture(ksfixtures.KeyRepository(self.config_fixture))
+            self.useFixture(
+                ksfixtures.KeyRepository(
+                    self.config_fixture,
+                    'fernet_tokens',
+                    CONF.fernet_tokens.max_active_keys
+                )
+            )
 
             # Validate the initial repository state.
             self.assertRepositoryState(expected_size=min_active_keys)
@@ -564,8 +601,13 @@ class TestFernetKeyRotation(unit.TestCase):
 
             # Rotate the keys just enough times to fully populate the key
             # repository.
+            key_utils = fernet_utils.FernetUtils(
+                CONF.fernet_tokens.key_repository,
+                CONF.fernet_tokens.max_active_keys,
+                'fernet_tokens'
+            )
             for rotation in range(max_active_keys - min_active_keys):
-                fernet_utils.rotate_keys()
+                key_utils.rotate_keys()
                 self.assertRepositoryState(expected_size=rotation + 3)
 
                 exp_keys.append(next_key_number)
@@ -577,8 +619,13 @@ class TestFernetKeyRotation(unit.TestCase):
 
             # Rotate an additional number of times to ensure that we maintain
             # the desired number of active keys.
+            key_utils = fernet_utils.FernetUtils(
+                CONF.fernet_tokens.key_repository,
+                CONF.fernet_tokens.max_active_keys,
+                'fernet_tokens'
+            )
             for rotation in range(10):
-                fernet_utils.rotate_keys()
+                key_utils.rotate_keys()
                 self.assertRepositoryState(expected_size=max_active_keys)
 
                 exp_keys.pop(1)
@@ -586,12 +633,53 @@ class TestFernetKeyRotation(unit.TestCase):
                 next_key_number += 1
                 self.assertEqual(exp_keys, self.keys)
 
+    def test_rotation_disk_write_fail(self):
+        # Init the key repository
+        self.useFixture(
+            ksfixtures.KeyRepository(
+                self.config_fixture,
+                'fernet_tokens',
+                CONF.fernet_tokens.max_active_keys
+            )
+        )
+
+        # Make sure that the init key repository contains 2 keys
+        self.assertRepositoryState(expected_size=2)
+
+        key_utils = fernet_utils.FernetUtils(
+            CONF.fernet_tokens.key_repository,
+            CONF.fernet_tokens.max_active_keys,
+            'fernet_tokens'
+        )
+
+        # Simulate the disk full situation
+        mock_open = mock.mock_open()
+        file_handle = mock_open()
+        file_handle.flush.side_effect = IOError('disk full')
+
+        with mock.patch('keystone.common.fernet_utils.open', mock_open):
+            self.assertRaises(IOError, key_utils.rotate_keys)
+
+        # Assert that the key repository is unchanged
+        self.assertEqual(self.key_repository_size, 2)
+
     def test_non_numeric_files(self):
-        self.useFixture(ksfixtures.KeyRepository(self.config_fixture))
+        self.useFixture(
+            ksfixtures.KeyRepository(
+                self.config_fixture,
+                'fernet_tokens',
+                CONF.fernet_tokens.max_active_keys
+            )
+        )
         evil_file = os.path.join(CONF.fernet_tokens.key_repository, '99.bak')
         with open(evil_file, 'w'):
             pass
-        fernet_utils.rotate_keys()
+        key_utils = fernet_utils.FernetUtils(
+            CONF.fernet_tokens.key_repository,
+            CONF.fernet_tokens.max_active_keys,
+            'fernet_tokens'
+        )
+        key_utils.rotate_keys()
         self.assertTrue(os.path.isfile(evil_file))
         keys = 0
         for x in os.listdir(CONF.fernet_tokens.key_repository):
@@ -602,11 +690,27 @@ class TestFernetKeyRotation(unit.TestCase):
 
 
 class TestLoadKeys(unit.TestCase):
+
+    def assertValidFernetKey(self, keys):
+        self.assertGreater(len(keys[0]), 0)
+        self.assertIsInstance(keys[0], str)
+
     def test_non_numeric_files(self):
-        self.useFixture(ksfixtures.KeyRepository(self.config_fixture))
+        self.useFixture(
+            ksfixtures.KeyRepository(
+                self.config_fixture,
+                'fernet_tokens',
+                CONF.fernet_tokens.max_active_keys
+            )
+        )
         evil_file = os.path.join(CONF.fernet_tokens.key_repository, '~1')
         with open(evil_file, 'w'):
             pass
-        keys = fernet_utils.load_keys()
+        key_utils = fernet_utils.FernetUtils(
+            CONF.fernet_tokens.key_repository,
+            CONF.fernet_tokens.max_active_keys,
+            'fernet_tokens'
+        )
+        keys = key_utils.load_keys()
         self.assertEqual(2, len(keys))
-        self.assertTrue(len(keys[0]))
+        self.assertValidFernetKey(keys)
