@@ -15,25 +15,33 @@
 
 from __future__ import absolute_import
 import copy
+import threading
 
-from oslo_config import cfg
 from oslo_log import log
+from oslo_log import versionutils
 from oslo_utils import timeutils
 import six
 
 from keystone.common import kvs
 from keystone.common import utils
+import keystone.conf
 from keystone import exception
 from keystone.i18n import _, _LE, _LW
 from keystone import token
-from keystone.token import provider
+from keystone.token.providers import common
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 LOG = log.getLogger(__name__)
 
+STORE_CONF_LOCK = threading.Lock()
 
-class Token(token.persistence.TokenDriverV8):
+
+@versionutils.deprecated(
+    versionutils.deprecated.OCATA,
+    what='keystone.token.persistence.backends.kvs.Token',
+    remove_in=+1)
+class Token(token.persistence.TokenDriverBase):
     """KeyValueStore backend for tokens.
 
     This is the base implementation for any/all key-value-stores (e.g.
@@ -49,9 +57,12 @@ class Token(token.persistence.TokenDriverV8):
         self._store = kvs.get_key_value_store('token-driver')
         if backing_store is not None:
             self.kvs_backend = backing_store
-        if not self._store.is_configured:
-            # Do not re-configure the backend if the store has been initialized
-            self._store.configure(backing_store=self.kvs_backend, **kwargs)
+        # Using a lock here to avoid race condition.
+        with STORE_CONF_LOCK:
+            if not self._store.is_configured:
+                # Do not re-configure the backend if the store has been
+                # initialized.
+                self._store.configure(backing_store=self.kvs_backend, **kwargs)
         if self.__class__ == Token:
             # NOTE(morganfainberg): Only warn if the base KVS implementation
             # is instantiated.
@@ -61,10 +72,14 @@ class Token(token.persistence.TokenDriverV8):
                             "'memcache' or 'sql' instead."))
 
     def _prefix_token_id(self, token_id):
-        return 'token-%s' % token_id.encode('utf-8')
+        if six.PY2:
+            token_id = token_id.encode('utf-8')
+        return 'token-%s' % token_id
 
     def _prefix_user_id(self, user_id):
-        return 'usertokens-%s' % user_id.encode('utf-8')
+        if six.PY2:
+            user_id = user_id.encode('utf-8')
+        return 'usertokens-%s' % user_id
 
     def _get_key_or_default(self, key, default=None):
         try:
@@ -99,7 +114,7 @@ class Token(token.persistence.TokenDriverV8):
         data_copy = copy.deepcopy(data)
         ptk = self._prefix_token_id(token_id)
         if not data_copy.get('expires'):
-            data_copy['expires'] = provider.default_expire_time()
+            data_copy['expires'] = common.default_expire_time()
         if not data_copy.get('user_id'):
             data_copy['user_id'] = data_copy['user']['id']
 
@@ -164,24 +179,24 @@ class Token(token.persistence.TokenDriverV8):
             for item in token_list:
                 try:
                     item_id, expires = self._format_token_index_item(item)
-                except (ValueError, TypeError):
+                except (ValueError, TypeError):  # nosec(tkelsey)
                     # NOTE(morganfainberg): Skip on expected errors
                     # possibilities from the `_format_token_index_item` method.
                     continue
 
                 if expires < current_time:
-                    LOG.debug(('Token `%(token_id)s` is expired, removing '
-                               'from `%(user_key)s`.'),
-                              {'token_id': item_id, 'user_key': user_key})
+                    msg = ('Token `%(token_id)s` is expired, '
+                           'removing from `%(user_key)s`.')
+                    LOG.debug(msg, {'token_id': item_id, 'user_key': user_key})
                     continue
 
                 if item_id in revoked_token_list:
                     # NOTE(morganfainberg): If the token has been revoked, it
                     # can safely be removed from this list.  This helps to keep
                     # the user_token_list as reasonably small as possible.
-                    LOG.debug(('Token `%(token_id)s` is revoked, removing '
-                               'from `%(user_key)s`.'),
-                              {'token_id': item_id, 'user_key': user_key})
+                    msg = ('Token `%(token_id)s` is revoked, removing '
+                           'from `%(user_key)s`.')
+                    LOG.debug(msg, {'token_id': item_id, 'user_key': user_key})
                     continue
                 filtered_list.append(item)
             filtered_list.append((token_id, expires_isotime_str))
@@ -328,7 +343,7 @@ class Token(token.persistence.TokenDriverV8):
         for item in token_list:
             try:
                 token_id, expires = self._format_token_index_item(item)
-            except (TypeError, ValueError):
+            except (TypeError, ValueError):  # nosec(tkelsey)
                 # NOTE(morganfainberg): Skip on expected error possibilities
                 # from the `_format_token_index_item` method.
                 continue
@@ -338,7 +353,7 @@ class Token(token.persistence.TokenDriverV8):
 
             try:
                 token_ref = self.get_token(token_id)
-            except exception.TokenNotFound:
+            except exception.TokenNotFound:  # nosec(tkelsey)
                 # NOTE(morganfainberg): Token doesn't exist, skip it.
                 continue
             if token_ref:
