@@ -12,14 +12,12 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import time
-
 from oslo_utils import timeutils
 from six.moves import range
 
 from keystone.common import sql
 from keystone import exception
-from keystone import trust
+from keystone.trust.backends import base
 
 
 # The maximum number of iterations that will be attempted for optimistic
@@ -56,7 +54,7 @@ class TrustRole(sql.ModelBase):
     role_id = sql.Column(sql.String(64), primary_key=True, nullable=False)
 
 
-class Trust(trust.TrustDriverV8):
+class Trust(base.TrustDriverBase):
     @sql.handle_conflicts(conflict_type='trust')
     def create_trust(self, trust_id, trust, roles):
         with sql.session_for_write() as session:
@@ -82,9 +80,7 @@ class Trust(trust.TrustDriverV8):
             roles.append({'id': role.role_id})
         trust_dict['roles'] = roles
 
-    @sql.handle_conflicts(conflict_type='trust')
     def consume_use(self, trust_id):
-
         for attempt in range(MAXIMUM_CONSUME_ATTEMPTS):
             with sql.session_for_write() as session:
                 try:
@@ -118,10 +114,6 @@ class Trust(trust.TrustDriverV8):
                         break
                 else:
                     raise exception.TrustUseLimitReached(trust_id=trust_id)
-            # NOTE(morganfainberg): Ensure we have a yield point for eventlet
-            # here. This should cost us nothing otherwise. This can be removed
-            # if/when oslo_db cleanly handles yields on db calls.
-            time.sleep(0)
         else:
             # NOTE(morganfainberg): In the case the for loop is not prematurely
             # broken out of, this else block is executed. This means the trust
@@ -152,13 +144,11 @@ class Trust(trust.TrustDriverV8):
             self._add_roles(trust_id, session, trust_dict)
             return trust_dict
 
-    @sql.handle_conflicts(conflict_type='trust')
     def list_trusts(self):
         with sql.session_for_read() as session:
             trusts = session.query(TrustModel).filter_by(deleted_at=None)
             return [trust_ref.to_dict() for trust_ref in trusts]
 
-    @sql.handle_conflicts(conflict_type='trust')
     def list_trusts_for_trustee(self, trustee_user_id):
         with sql.session_for_read() as session:
             trusts = (session.query(TrustModel).
@@ -166,7 +156,6 @@ class Trust(trust.TrustDriverV8):
                       filter_by(trustee_user_id=trustee_user_id))
             return [trust_ref.to_dict() for trust_ref in trusts]
 
-    @sql.handle_conflicts(conflict_type='trust')
     def list_trusts_for_trustor(self, trustor_user_id):
         with sql.session_for_read() as session:
             trusts = (session.query(TrustModel).
@@ -181,3 +170,10 @@ class Trust(trust.TrustDriverV8):
             if not trust_ref:
                 raise exception.TrustNotFound(trust_id=trust_id)
             trust_ref.deleted_at = timeutils.utcnow()
+
+    def delete_trusts_for_project(self, project_id):
+        with sql.session_for_write() as session:
+            query = session.query(TrustModel)
+            trusts = query.filter_by(project_id=project_id)
+            for trust_ref in trusts:
+                trust_ref.deleted_at = timeutils.utcnow()
